@@ -11,10 +11,11 @@
 #include "option_change_event_handler.h"
 
 #include <atomic>
+#include <list>
+#include <deque>
 
 class CControlSocket;
 class CLogging;
-class CRateLimiter;
 class OpLockManager;
 
 enum EngineNotificationType
@@ -63,9 +64,6 @@ public:
 	int Cancel();
 	int ResetOperation(int nErrorCode);
 
-	const CCommand *GetCurrentCommand() const;
-	Command GetCurrentCommandId() const;
-
 	bool IsBusy() const;
 	bool IsConnected() const;
 
@@ -87,11 +85,12 @@ public:
 	std::unique_ptr<CNotification> GetNextNotification();
 
 	COptionsBase& GetOptions() { return m_options; }
-	CRateLimiter& GetRateLimiter() { return m_rateLimiter; }
+	fz::rate_limiter& GetRateLimiter() { return rate_limiter_; }
 	CDirectoryCache& GetDirectoryCache() { return directory_cache_; }
 	CPathCache& GetPathCache() { return path_cache_; }
 	fz::thread_pool& GetThreadPool() { return thread_pool_; }
 	CFileZillaEngineContext& GetContext() { return context_; }
+	CFileZillaEngine& GetParent() { return parent_; }
 
 	// If deleting or renaming a directory, it could be possible that another
 	// engine's CControlSocket instance still has that directory as
@@ -168,16 +167,13 @@ protected:
 
 	std::unique_ptr<CControlSocket> controlSocket_;
 
-	std::unique_ptr<CCommand> m_pCurrentCommand;
+	std::unique_ptr<CCommand> currentCommand_;
 
 	// Protect access to these three with notification_mutex_
 	std::deque<CNotification*> m_NotificationList;
 	bool m_maySendNotificationEvent{true};
 	unsigned int m_asyncRequestCounter{};
 
-	bool m_bIsInCommand{}; //true if Command is on the callstack
-	int m_nControlSocketError{};
-
 	COptionsBase& m_options;
 
 	std::unique_ptr<CLogging> logger_;
@@ -200,7 +196,7 @@ protected:
 	int m_retryCount{};
 	fz::timer_id m_retryTimer{};
 
-	CRateLimiter& m_rateLimiter;
+	fz::rate_limiter& rate_limiter_;
 	CDirectoryCache& directory_cache_;
 	CPathCache& path_cache_;
 
@@ -217,9 +213,6 @@ protected:
 	CFileZillaEngineContext& context_;
 };
 
-struct command_event_type{};
-typedef fz::simple_event<command_event_type> CCommandEvent;
-
 struct async_request_reply_event_type{};
 typedef fz::simple_event<async_request_reply_event_type, std::unique_ptr<CAsyncRequestNotification>> CAsyncRequestReplyEvent;
 
