@@ -93,7 +93,7 @@ void CFtpControlSocket::OnReceive()
 					ParseLine(line);
 
 					// Abort if connection got closed
-					if (!currentServer_) {
+					if (!active_layer_) {
 						return;
 					}
 				}
@@ -105,6 +105,7 @@ void CFtpControlSocket::OnReceive()
 		if (receiveBuffer_.size() == max) {
 			log(fz::logmsg::error, _("Received too long response line from server, closing connection."));
 			DoClose();
+			return;
 		}
 	}
 }
@@ -173,6 +174,7 @@ void CFtpControlSocket::ParseLine(std::wstring line)
 void CFtpControlSocket::OnConnect()
 {
 	m_lastTypeBinary = -1;
+	m_sentRestartOffset = false;
 
 	SetAlive();
 
@@ -202,7 +204,6 @@ void CFtpControlSocket::OnConnect()
 		log(logmsg::status, _("Connection established, waiting for welcome message..."));
 	}
 	m_pendingReplies = 1;
-	m_repliesToSkip = 0;
 }
 
 void CFtpControlSocket::ParseResponse()
@@ -312,21 +313,6 @@ int CFtpControlSocket::SendCommand(std::wstring const& str, bool maskArgs, bool
 
 void CFtpControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		log(logmsg::status, _("Retrieving directory listing..."));
-	}
-	else {
-		log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
 	Push(std::make_unique<CFtpListOpData>(*this, path, subDir, flags));
 }
 
@@ -431,7 +417,6 @@ void CFtpControlSocket::FileTransfer(std::wstring const& localFile, CServerPath
 	log(logmsg::debug_verbose, L"CFtpControlSocket::FileTransfer()");
 
 	auto pData = std::make_unique<CFtpFileTransferOpData>(*this, download, localFile, remoteFile, remotePath, transferSettings);
-	pData->binary = transferSettings.binary;
 	Push(std::move(pData));
 }
 
@@ -505,14 +490,12 @@ bool CFtpControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotific
 				return false;
 			}
 
-			auto & data = static_cast<CFtpLogonOpData&>(*operations_.back());
-
 			CInteractiveLoginNotification *pInteractiveLoginNotification = static_cast<CInteractiveLoginNotification *>(pNotification);
 			if (!pInteractiveLoginNotification->passwordSet) {
 				ResetOperation(FZ_REPLY_CANCELED);
 				return false;
 			}
-			data.credentials_.SetPass(pInteractiveLoginNotification->credentials.GetPass());
+			credentials_.SetPass(pInteractiveLoginNotification->credentials.GetPass());
 			SendNextCommand();
 		}
 		break;
@@ -587,15 +570,6 @@ void CFtpControlSocket::RemoveDir(CServerPath const& path, std::wstring const& s
 
 void CFtpControlSocket::Mkdir(CServerPath const& path)
 {
-	/* Directory creation works like this: First find a parent directory into
-	 * which we can CWD, then create the subdirs one by one. If either part
-	 * fails, try MKD with the full path directly.
-	 */
-
-	if (operations_.empty() && !path.empty()) {
-		log(logmsg::status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CFtpMkdirOpData>(*this);
 	pData->path_ = path;
 
@@ -604,15 +578,11 @@ void CFtpControlSocket::Mkdir(CServerPath const& path)
 
 void CFtpControlSocket::Rename(CRenameCommand const& command)
 {
-	log(logmsg::status, _("Renaming '%s' to '%s'"), command.GetFromPath().FormatFilename(command.GetFromFile()), command.GetToPath().FormatFilename(command.GetToFile()));
-
 	Push(std::make_unique<CFtpRenameOpData>(*this, command));
 }
 
 void CFtpControlSocket::Chmod(CChmodCommand const& command)
 {
-	log(logmsg::status, _("Setting permissions of '%s' to '%s'"), command.GetPath().FormatFilename(command.GetFile()), command.GetPermission());
-
 	Push(std::make_unique<CFtpChmodOpData>(*this, command));
 }
 
@@ -711,39 +681,6 @@ void CFtpControlSocket::Transfer(std::wstring const& cmd, CFtpTransferOpData* ol
 	pData->pOldData = oldData;
 	pData->pOldData->transferEndReason = TransferEndReason::successful;
 
-	if (proxy_layer_) {
-		// Only passive suported
-		// Theoretically could use reverse proxy ability in SOCKS5, but
-		// it is too fragile to set up with all those broken routers and
-		// firewalls sabotaging connections. Regular active mode is hard
-		// enough already
-		pData->bPasv = true;
-		pData->bTriedActive = true;
-	}
-	else {
-		switch (currentServer_.GetPasvMode())
-		{
-		case MODE_PASSIVE:
-			pData->bPasv = true;
-			break;
-		case MODE_ACTIVE:
-			pData->bPasv = false;
-			break;
-		default:
-			pData->bPasv = engine_.GetOptions().GetOptionVal(OPTION_USEPASV) != 0;
-			break;
-		}
-	}
-
-	if ((pData->pOldData->binary && m_lastTypeBinary == 1) ||
-		(!pData->pOldData->binary && m_lastTypeBinary == 0))
-	{
-		pData->opState = rawtransfer_port_pasv;
-	}
-	else {
-		pData->opState = rawtransfer_type;
-	}
-
 	Push(std::move(pData));
 }
 
@@ -755,8 +692,9 @@ void CFtpControlSocket::Connect(CServer const& server, Credentials const& creden
 	}
 
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	Push(std::make_unique<CFtpLogonOpData>(*this, credentials));
+	Push(std::make_unique<CFtpLogonOpData>(*this));
 }
 
 void CFtpControlSocket::OnTimer(fz::timer_id id)
@@ -850,6 +788,7 @@ void CFtpControlSocket::ResetSocket()
 {
 	receiveBuffer_.clear();
 	tls_layer_.reset();
+	m_pendingReplies = 0;
 	CRealControlSocket::ResetSocket();
 }
 
@@ -861,3 +800,15 @@ void CFtpControlSocket::OnVerifyCert(fz::tls_layer* source, fz::tls_session_info
 
 	SendAsyncRequest(new CCertificateNotification(std::move(info)));
 }
+
+void CFtpControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CRealControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!active_layer_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CFtpLogonOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CRealControlSocket::Push(std::move(connOp));
+		}
+	}
+}
