@@ -2,7 +2,7 @@
 #define FILEZILLA_ENGINE_FTP_FTPCONTROLSOCKET_HEADER
 
 #include "logging_private.h"
-#include "ControlSocket.h"
+#include "controlsocket.h"
 #include "externalipresolver.h"
 #include "rtt.h"
 
@@ -35,6 +35,8 @@ public:
 
 protected:
 
+	virtual void Push(std::unique_ptr<COpData> && pNewOpData) override;
+
 	virtual int ResetOperation(int nErrorCode) override;
 
 	// Implicit FZ_REPLY_CONTINUE
@@ -45,7 +47,7 @@ protected:
 							 std::wstring const& remoteFile, bool download,
 							 CFileTransferCommand::t_transferSettings const& transferSettings) override;
 	virtual void RawCommand(std::wstring const& command) override;
-	virtual void Delete(CServerPath const& path, std::deque<std::wstring>&& files) override;
+	virtual void Delete(CServerPath const& path, std::vector<std::wstring>&& files) override;
 	virtual void RemoveDir(CServerPath const& path, std::wstring const& subDir) override;
 	virtual void Mkdir(CServerPath const& path) override;
 	virtual void Rename(CRenameCommand const& command) override;
