@@ -48,6 +48,14 @@ CFtpListOpData::CFtpListOpData(CFtpControlSocket & controlSocket, CServerPath co
 int CFtpListOpData::Send()
 {
 	if (opState == list_init) {
+		auto newPath = CServerPath::GetChanged(currentPath_, path_, subDir_);
+		if (newPath.empty()) {
+			log(logmsg::status, _("Retrieving directory listing..."));
+		}
+		else {
+			log(logmsg::status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
+		}
+
 		controlSocket_.ChangeDir(path_, subDir_, (flags_ & LIST_FLAG_LINK));
 		opState = list_waitcwd;
 		return FZ_REPLY_CONTINUE;
@@ -104,7 +112,7 @@ int CFtpListOpData::Send()
 					viewHidden_ = true;
 				}
 				else {
-					LogMessage(MessageType::Debug_Info, _("View hidden option set, but unsupported by server"));
+					log(logmsg::debug_info, _("View hidden option set, but unsupported by server"));
 				}
 			}
 
@@ -118,12 +126,12 @@ int CFtpListOpData::Send()
 		return FZ_REPLY_CONTINUE;
 	}
 	if (opState == list_mdtm) {
-		LogMessage(MessageType::Status, _("Calculating timezone offset of server..."));
+		log(logmsg::status, _("Calculating timezone offset of server..."));
 		std::wstring cmd = L"MDTM " + currentPath_.FormatFilename(directoryListing_[mdtm_index_].name, true);
 		return controlSocket_.SendCommand(cmd);
 	}
 
-	LogMessage(MessageType::Debug_Warning, L"invalid opstate %d", opState);
+	log(logmsg::debug_warning, L"invalid opstate %d", opState);
 	return FZ_REPLY_INTERNALERROR;
 }
 
@@ -131,7 +139,7 @@ int CFtpListOpData::Send()
 int CFtpListOpData::ParseResponse()
 {
 	if (opState != list_mdtm) {
-		LogMessage(MessageType::Debug_Warning, "CFtpListOpData::ParseResponse should never be called if opState != list_mdtm");
+		log(logmsg::debug_warning, "CFtpListOpData::ParseResponse should never be called if opState != list_mdtm");
 		return FZ_REPLY_INTERNALERROR;
 	}
 
@@ -156,11 +164,11 @@ int CFtpListOpData::ParseResponse()
 				serveroffset -= serveroffset % 60;
 			}
 
-			LogMessage(MessageType::Status, L"Timezone offset of server is %d seconds.", -serveroffset);
+			log(logmsg::status, L"Timezone offset of server is %d seconds.", -serveroffset);
 
 			fz::duration span = fz::duration::from_seconds(serveroffset);
-			const int count = directoryListing_.GetCount();
-			for (int i = 0; i < count; ++i) {
+			size_t const count = directoryListing_.size();
+			for (size_t i = 0; i < count; ++i) {
 				CDirentry& entry = directoryListing_.get(i);
 				entry.time += span;
 			}
@@ -234,11 +242,11 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 				}
 				else {
 					if (CheckInclusion(listing, directoryListing_)) {
-						LogMessage(MessageType::Debug_Info, L"Server seems to support LIST -a");
+						log(logmsg::debug_info, L"Server seems to support LIST -a");
 						CServerCapabilities::SetCapability(currentServer_, list_hidden_support, yes);
 					}
 					else {
-						LogMessage(MessageType::Debug_Info, L"Server does not seem to support LIST -a");
+						log(logmsg::debug_info, L"Server does not seem to support LIST -a");
 						CServerCapabilities::SetCapability(currentServer_, list_hidden_support, no);
 						listing = directoryListing_;
 					}
@@ -266,15 +274,15 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 
 				if (viewHiddenCheck_) {
 					if (viewHidden_) {
-						if (directoryListing_.GetCount()) {
+						if (directoryListing_.size()) {
 							// Less files with LIST -a
 							// Not supported
-							LogMessage(MessageType::Debug_Info, L"Server does not seem to support LIST -a");
+							log(logmsg::debug_info, L"Server does not seem to support LIST -a");
 							CServerCapabilities::SetCapability(currentServer_, list_hidden_support, no);
 							listing = directoryListing_;
 						}
 						else {
-							LogMessage(MessageType::Debug_Info, L"Server seems to support LIST -a");
+							log(logmsg::debug_info, L"Server seems to support LIST -a");
 							CServerCapabilities::SetCapability(currentServer_, list_hidden_support, yes);
 						}
 					}
@@ -338,7 +346,7 @@ int CFtpListOpData::SubcommandResult(int prevResult, COpData const&)
 		}
 	}
 	else {
-		LogMessage(MessageType::Debug_Warning, L"Wrong opState: %d", opState);
+		log(logmsg::debug_warning, L"Wrong opState: %d", opState);
 		return FZ_REPLY_INTERNALERROR;
 	}
 }
@@ -350,8 +358,8 @@ int CFtpListOpData::CheckTimezoneDetection(CDirectoryListing& listing)
 			CServerCapabilities::SetCapability(currentServer_, timezone_offset, no);
 		}
 		else {
-			const int count = listing.GetCount();
-			for (int i = 0; i < count; ++i) {
+			size_t const count = listing.size();
+			for (size_t i = 0; i < count; ++i) {
 				if (!listing[i].is_dir() && listing[i].has_time()) {
 					opState = list_mdtm;
 					directoryListing_ = listing;
