@@ -3,12 +3,12 @@
 #include "logon.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
-#include "../tlssocket.h"
 
-CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket, Credentials const& credentials)
+#include <libfilezilla/tls_layer.hpp>
+
+CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket)
 	: COpData(Command::connect, L"CFtpLogonOpData")
 	, CFtpOpData(controlSocket)
-	, credentials_(credentials)
 {
 	for (int i = 0; i < LOGON_DONE; ++i) {
 		neededCommands[i] = 1;
@@ -47,11 +47,11 @@ int CFtpLogonOpData::Send()
 			if ((generic_proxy_type <= static_cast<int>(ProxyType::NONE) || generic_proxy_type >= static_cast<int>(ProxyType::count)) && !currentServer_.GetBypassProxy()) {
 				ftp_proxy_type_ = engine_.GetOptions().GetOptionVal(OPTION_FTP_PROXY_TYPE);
 			}
-		
+
 			if (!PrepareLoginSequence()) {
 				return FZ_REPLY_INTERNALERROR;
 			}
-			
+
 			if (ftp_proxy_type_) {
 				host_ = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_HOST);
 
@@ -60,12 +60,12 @@ int CFtpLogonOpData::Send()
 					// Probably IPv6 address
 					pos = host_.find(']');
 					if (pos == std::wstring::npos) {
-						LogMessage(MessageType::Error, _("Proxy host starts with '[' but no closing bracket found."));
+						log(logmsg::error, _("Proxy host starts with '[' but no closing bracket found."));
 						return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 					}
 					if (host_.size() > (pos + 1) && host_[pos + 1]) {
 						if (host_[pos + 1] != ':') {
-							LogMessage(MessageType::Error, _("Invalid proxy host, after closing bracket only colon and port may follow."));
+							log(logmsg::error, _("Invalid proxy host, after closing bracket only colon and port may follow."));
 							return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 						}
 						++pos;
@@ -87,11 +87,11 @@ int CFtpLogonOpData::Send()
 				}
 
 				if (host_.empty() || port_ < 1 || port_ > 65535) {
-					LogMessage(MessageType::Error, _("Proxy set but proxy host or port invalid"));
+					log(logmsg::error, _("Proxy set but proxy host or port invalid"));
 					return FZ_REPLY_DISCONNECTED | FZ_REPLY_CRITICALERROR;
 				}
 
-				LogMessage(MessageType::Status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
+				log(logmsg::status, _("Connecting to %s through %s proxy"), currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_), L"FTP"); // @translator: Connecting to ftp.example.com through SOCKS5 proxy
 			}
 			else {
 				host_ = currentServer_.GetHost();
@@ -102,7 +102,7 @@ int CFtpLogonOpData::Send()
 			int ret = controlSocket_.DoConnect(host_, port_);
 			if (ret == FZ_REPLY_WOULDBLOCK) {
 				// Enable TCP_NODELAY, speeds things up a bit.
-				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive);
+				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay | fz::socket::flag_keepalive, true);
 
 				// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
 				// firewalls which terminate the control connection on long transfers.
@@ -114,12 +114,25 @@ int CFtpLogonOpData::Send()
 			return ret;
 		}
 	case LOGON_AUTH_WAIT:
-		LogMessage(MessageType::Debug_Info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
+		log(logmsg::debug_info, L"LogonSend() called during LOGON_AUTH_WAIT, ignoring");
 		return FZ_REPLY_WOULDBLOCK;
 	case LOGON_AUTH_TLS:
 		return controlSocket_.SendCommand(L"AUTH TLS", false, false);
 	case LOGON_AUTH_SSL:
 		return controlSocket_.SendCommand(L"AUTH SSL", false, false);
+	case LOGON_SECURITY:
+		if (currentServer_.GetProtocol() == INSECURE_FTP) {
+			log(logmsg::status, _("Plain FTP is insecure. Please switch to FTP over TLS."));
+		}
+		opState = LOGON_LOGON;
+		if (controlSocket_.tls_layer_) {
+			return FZ_REPLY_CONTINUE;
+		}
+		else {
+			auto notification = new CInsecureConnectionNotification(currentServer_);
+			controlSocket_.SendAsyncRequest(notification);
+			return FZ_REPLY_WOULDBLOCK;
+		}
 	case LOGON_SYST:
 		return controlSocket_.SendCommand(L"SYST");
 	case LOGON_LOGON:
@@ -128,13 +141,13 @@ int CFtpLogonOpData::Send()
 			switch (cmd.type)
 			{
 			case loginCommandType::user:
-				if (credentials_.logonType_ == LogonType::interactive) {
+				if (controlSocket_.credentials_.logonType_ == LogonType::interactive) {
 					waitChallenge = true;
 					challenge.clear();
 				}
 
 				if (cmd.command.empty()) {
-					std::wstring const user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+					std::wstring const user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
 					return controlSocket_.SendCommand(L"USER " + user);
 				}
 				else {
@@ -145,7 +158,7 @@ int CFtpLogonOpData::Send()
 					CInteractiveLoginNotification *pNotification = new CInteractiveLoginNotification(CInteractiveLoginNotification::interactive, challenge, false);
 					pNotification->server = currentServer_;
 					pNotification->handle_ = controlSocket_.handle_;
-					pNotification->credentials = credentials_;
+					pNotification->credentials = controlSocket_.credentials_;
 					challenge.clear();
 
 					controlSocket_.SendAsyncRequest(pNotification);
@@ -153,7 +166,7 @@ int CFtpLogonOpData::Send()
 					return FZ_REPLY_WOULDBLOCK;
 				}
 				else {
-					std::wstring pass = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : credentials_.GetPass();
+					std::wstring pass = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : controlSocket_.credentials_.GetPass();
 					if (cmd.command.empty()) {
 						return controlSocket_.SendCommand(L"PASS " + pass, true);
 					}
@@ -168,7 +181,7 @@ int CFtpLogonOpData::Send()
 				break;
 			case loginCommandType::account:
 				if (cmd.command.empty()) {
-					return controlSocket_.SendCommand(L"ACCT " + credentials_.account_);
+					return controlSocket_.SendCommand(L"ACCT " + controlSocket_.credentials_.account_);
 				}
 				else {
 					return controlSocket_.SendCommand(cmd.command);
@@ -204,7 +217,7 @@ int CFtpLogonOpData::Send()
 		return controlSocket_.SendCommand(L"PROT P");
 	case LOGON_CUSTOMCOMMANDS:
 		if (customCommandIndex >= currentServer_.GetPostLoginCommands().size()) {
-			LogMessage(MessageType::Debug_Warning, L"pData->customCommandIndex >= m_pCurrentServer->GetPostLoginCommands().size()");
+			log(logmsg::debug_warning, L"pData->customCommandIndex >= m_pCurrentServer->GetPostLoginCommands().size()");
 			return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 		}
 		return controlSocket_.SendCommand(currentServer_.GetPostLoginCommands()[customCommandIndex]);
@@ -216,7 +229,7 @@ int CFtpLogonOpData::Send()
 		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"unknown op state: %d", opState);
+		log(logmsg::debug_warning, L"unknown op state: %d", opState);
 		break;
 	}
 
@@ -240,14 +253,11 @@ int CFtpLogonOpData::ParseResponse()
 			CServerCapabilities::SetCapability(currentServer_, (opState == LOGON_AUTH_TLS) ? auth_tls_command : auth_ssl_command, no);
 			if (opState == LOGON_AUTH_SSL) {
 				if (currentServer_.GetProtocol() == FTP) {
-					LogMessage(MessageType::Status, _("Insecure server, it does not support FTP over TLS."));
+					log(logmsg::status, _("Insecure server, it does not support FTP over TLS."));
 					neededCommands[LOGON_PBSZ] = 0;
 					neededCommands[LOGON_PROT] = 0;
-
-					auto notification = new CInsecureFTPNotification(currentServer_);
-					controlSocket_.SendAsyncRequest(notification);
-					opState = LOGON_LOGON;
-					return FZ_REPLY_WOULDBLOCK;
+					opState = LOGON_SECURITY;
+					return FZ_REPLY_CONTINUE;
 				}
 				else {
 					return FZ_REPLY_DISCONNECTED | (code == 5 ? FZ_REPLY_CRITICALERROR : FZ_REPLY_ERROR);
@@ -257,12 +267,12 @@ int CFtpLogonOpData::ParseResponse()
 		else {
 			CServerCapabilities::SetCapability(currentServer_, (opState == LOGON_AUTH_TLS) ? auth_tls_command : auth_ssl_command, yes);
 
-			LogMessage(MessageType::Status, _("Initializing TLS..."));
+			log(logmsg::status, _("Initializing TLS..."));
 
-			controlSocket_.tls_layer_ = std::make_unique<CTlsSocket>(&controlSocket_, *controlSocket_.active_layer_, &controlSocket_);
+			controlSocket_.tls_layer_ = std::make_unique<fz::tls_layer>(controlSocket_.event_loop_, &controlSocket_, *controlSocket_.active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), controlSocket_.logger_);
 			controlSocket_.active_layer_ = controlSocket_.tls_layer_.get();
 
-			if (!controlSocket_.tls_layer_->client_handshake()) {
+			if (!controlSocket_.tls_layer_->client_handshake(&controlSocket_)) {
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 
@@ -279,11 +289,11 @@ int CFtpLogonOpData::ParseResponse()
 			if (cmd.type == loginCommandType::user || cmd.type == loginCommandType::pass) {
 				auto const user = currentServer_.GetUser();
 				if (!user.empty() && (user.front() == ' ' || user.back() == ' ')) {
-					LogMessage(MessageType::Status, _("Check your login credentials. The entered username starts or ends with a space character."));
+					log(logmsg::status, _("Check your login credentials. The entered username starts or ends with a space character."));
 				}
-				auto const pw = credentials_.GetPass();
+				auto const pw = controlSocket_.credentials_.GetPass();
 				if (!pw.empty() && (pw.front() == ' ' || pw.back() == ' ')) {
-					LogMessage(MessageType::Status, _("Check your login credentials. The entered password starts or ends with a space character."));
+					log(logmsg::status, _("Check your login credentials. The entered password starts or ends with a space character."));
 				}
 			}
 
@@ -294,22 +304,22 @@ int CFtpLogonOpData::ParseResponse()
 				if (!fz::str_is_ascii(currentServer_.GetUser())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(credentials_.GetPass())) {
+				if (!fz::str_is_ascii(controlSocket_.credentials_.GetPass())) {
 					asciiOnly = false;
 				}
-				if (!fz::str_is_ascii(credentials_.account_)) {
+				if (!fz::str_is_ascii(controlSocket_.credentials_.account_)) {
 					asciiOnly = false;
 				}
 				if (!asciiOnly) {
 					if (ftp_proxy_type_) {
-						LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Cannot fall back to local charset since using proxy."));
+						log(logmsg::status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Cannot fall back to local charset since using proxy."));
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
 						if (cmd.type == loginCommandType::pass && code == 5) {
 							error |= FZ_REPLY_PASSWORDFAILED;
 						}
 						return error;
 					}
-					LogMessage(MessageType::Status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Trying local charset."));
+					log(logmsg::status, _("Login data contains non-ASCII characters and server might not be UTF-8 aware. Trying local charset."));
 					controlSocket_.m_useUTF8 = false;
 					if (!PrepareLoginSequence()) {
 						int error = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR;
@@ -336,9 +346,9 @@ int CFtpLogonOpData::ParseResponse()
 			}
 		}
 		else if (code == 3 && loginSequence.empty()) {
-			LogMessage(MessageType::Error, _("Login sequence fully executed yet not logged in, aborting."));
-			if (cmd.type == loginCommandType::pass && credentials_.account_.empty()) {
-				LogMessage(MessageType::Error, _("Server might require an account. Try specifying an account using the Site Manager"));
+			log(logmsg::error, _("Login sequence fully executed yet not logged in, aborting."));
+			if (cmd.type == loginCommandType::pass && controlSocket_.credentials_.account_.empty()) {
+				log(logmsg::error, _("Server might require an account. Try specifying an account using the Site Manager"));
 			}
 			return FZ_REPLY_CRITICALERROR | FZ_REPLY_DISCONNECTED;
 		}
@@ -396,7 +406,7 @@ int CFtpLogonOpData::ParseResponse()
 
 		const CharsetEncoding encoding = currentServer_.GetEncodingType();
 		if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(currentServer_, utf8_command) != yes) {
-			LogMessage(MessageType::Status, _("Server does not support non-ASCII characters."));
+			log(logmsg::status, _("Server does not support non-ASCII characters."));
 			controlSocket_.m_useUTF8 = false;
 		}
 	}
@@ -416,8 +426,8 @@ int CFtpLogonOpData::ParseResponse()
 		++opState;
 
 		if (opState == LOGON_DONE) {
-			LogMessage(MessageType::Status, _("Logged in"));
-			LogMessage(MessageType::Debug_Info, L"Measured latency of %d ms", controlSocket_.m_rtt.GetLatency());
+			log(logmsg::status, _("Logged in"));
+			log(logmsg::debug_info, L"Measured latency of %d ms", controlSocket_.m_rtt.GetLatency());
 			return FZ_REPLY_OK;
 		}
 
@@ -456,7 +466,7 @@ int CFtpLogonOpData::ParseResponse()
 			}
 			const CharsetEncoding encoding = currentServer_.GetEncodingType();
 			if (encoding == ENCODING_AUTO && CServerCapabilities::GetCapability(currentServer_, utf8_command) != yes) {
-				LogMessage(MessageType::Status, _("Server does not support non-ASCII characters."));
+				log(logmsg::status, _("Server does not support non-ASCII characters."));
 				controlSocket_.m_useUTF8 = false;
 			}
 		}
@@ -576,7 +586,7 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
@@ -594,8 +604,8 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 			loginSequence.push_back(cmd);
 		}
 		// User@host
-		std::wstring const user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
-		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", user, currentServer_.Format(ServerFormat::with_optional_port, credentials_))};
+		std::wstring const user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+		t_loginCommand cmd = {false, false, loginCommandType::user, fz::sprintf(L"USER %s@%s", user, currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_))};
 		loginSequence.push_back(cmd);
 
 		// Password
@@ -606,7 +616,7 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
@@ -627,10 +637,10 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		// Site or Open
 		t_loginCommand cmd = {false, false, loginCommandType::user, L""};
 		if (ftp_proxy_type_ == 2) {
-			cmd.command = L"SITE " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
+			cmd.command = L"SITE " + currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
 		}
 		else {
-			cmd.command = L"OPEN " + currentServer_.Format(ServerFormat::with_optional_port, credentials_);
+			cmd.command = L"OPEN " + currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
 		}
 		loginSequence.push_back(cmd);
 
@@ -646,7 +656,7 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		loginSequence.push_back(cmd);
 
 		// Optional account
-		if (!credentials_.account_.empty()) {
+		if (!controlSocket_.credentials_.account_.empty()) {
 			cmd.hide_arguments = false;
 			cmd.type = loginCommandType::account;
 			loginSequence.push_back(cmd);
@@ -655,9 +665,9 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 	else if (ftp_proxy_type_ == 4) {
 		std::wstring proxyUser = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_USER);
 		std::wstring proxyPass = engine_.GetOptions().GetOption(OPTION_FTP_PROXY_PASS);
-		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, credentials_);
-		std::wstring user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
-		std::wstring account = credentials_.account_;
+		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
+		std::wstring user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+		std::wstring account = controlSocket_.credentials_.account_;
 		fz::replace_substrings(proxyUser, L"%", L"%%");
 		fz::replace_substrings(proxyPass, L"%", L"%%");
 		fz::replace_substrings(host, L"%", L"%%");
@@ -749,12 +759,12 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		}
 
 		if (loginSequence.empty()) {
-			LogMessage(MessageType::Error, _("Could not generate custom login sequence."));
+			log(logmsg::error, _("Could not generate custom login sequence."));
 			return false;
 		}
 	}
 	else {
-		LogMessage(MessageType::Error, _("Unknown FTP proxy type, cannot generate login sequence."));
+		log(logmsg::error, _("Unknown FTP proxy type, cannot generate login sequence."));
 		return false;
 	}
 
