@@ -5,6 +5,7 @@
 #include "input_thread.h"
 #include "proxy.h"
 
+#include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
 int CSftpConnectOpData::Send()
@@ -13,26 +14,49 @@ int CSftpConnectOpData::Send()
 	{
 	case connect_init:
 		{
+			log(logmsg::status, _("Connecting to %s..."), currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_));
+
+			if (!controlSocket_.credentials_.keyFile_.empty()) {
+				keyfiles_ = fz::strtok(controlSocket_.credentials_.keyFile_, L"\r\n");
+			}
+			else {
+				keyfiles_ = fz::strtok(engine_.GetOptions().GetOption(OPTION_SFTP_KEYFILES), L"\r\n");
+			}
+
+			keyfiles_.erase(
+						std::remove_if(keyfiles_.begin(), keyfiles_.end(),
+									   [this](std::wstring const& keyfile) {
+							if (fz::local_filesys::get_file_type(fz::to_native(keyfile), true) != fz::local_filesys::file) {
+								log(logmsg::status, _("Skipping non-existing key file \"%s\""), keyfile);
+								return true;
+							}
+							return false;
+						}), keyfiles_.end());
+
+			keyfile_ = keyfiles_.cbegin();
+
 			auto executable = fz::to_native(engine_.GetOptions().GetOption(OPTION_FZSFTP_EXECUTABLE));
 			if (executable.empty()) {
 				executable = fzT("fzsftp");
 			}
-			LogMessage(MessageType::Debug_Verbose, L"Going to execute %s", executable);
+			log(logmsg::debug_verbose, L"Going to execute %s", executable);
 
 			std::vector<fz::native_string> args = { fzT("-v") };
 			if (engine_.GetOptions().GetOptionVal(OPTION_SFTP_COMPRESSION)) {
 				args.push_back(fzT("-C"));
 			}
+			engine_.GetRateLimiter().add(&controlSocket_);
+			controlSocket_.process_ = std::make_unique<fz::process>();
 			if (!controlSocket_.process_->spawn(executable, args)) {
-				LogMessage(MessageType::Debug_Warning, L"Could not create process");
-				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;;
+				log(logmsg::debug_warning, L"Could not create process");
+				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 
 			controlSocket_.input_thread_ = std::make_unique<CSftpInputThread>(controlSocket_, *controlSocket_.process_);
 			if (!controlSocket_.input_thread_->spawn(engine_.GetThreadPool())) {
-				LogMessage(MessageType::Debug_Warning, L"Thread creation failed");
+				log(logmsg::debug_warning, L"Thread creation failed");
 				controlSocket_.input_thread_.reset();
-				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;;
+				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
 		}
 		return FZ_REPLY_WOULDBLOCK;
@@ -51,7 +75,7 @@ int CSftpConnectOpData::Send()
 				type = 3;
 				break;
 			default:
-				LogMessage(MessageType::Debug_Warning, L"Unsupported proxy type");
+				log(logmsg::debug_warning, L"Unsupported proxy type");
 				return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 			}
 
@@ -76,11 +100,11 @@ int CSftpConnectOpData::Send()
 		return controlSocket_.SendCommand(L"keyfile \"" + *(keyfile_++) + L"\"");
 	case connect_open:
 		{
-			std::wstring user = (credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
+			std::wstring user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
 			return controlSocket_.SendCommand(fz::sprintf(L"open \"%s@%s\" %d", user, controlSocket_.ConvertDomainName(currentServer_.GetHost()), currentServer_.GetPort()));
 		}
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown op state: %d", opState);
+		log(logmsg::debug_warning, L"Unknown op state: %d", opState);
 		break;
 	}
 
@@ -97,8 +121,8 @@ int CSftpConnectOpData::ParseResponse()
 	{
 	case connect_init:
 		if (controlSocket_.response_ != fz::sprintf(L"fzSftp started, protocol_version=%d", FZSFTP_PROTOCOL_VERSION)) {
-			LogMessage(MessageType::Error, _("fzsftp belongs to a different version of FileZilla"));
-			return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;;
+			log(logmsg::error, _("fzsftp belongs to a different version of FileZilla"));
+			return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 		}
 		if (engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE) && !currentServer_.GetBypassProxy()) {
 			opState = connect_proxy;
@@ -127,7 +151,7 @@ int CSftpConnectOpData::ParseResponse()
 		engine_.AddNotification(new CSftpEncryptionNotification(controlSocket_.m_sftpEncryptionDetails));
 		return FZ_REPLY_OK;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown op state: %d", opState);
+		log(logmsg::debug_warning, L"Unknown op state: %d", opState);
 		return FZ_REPLY_INTERNALERROR | FZ_REPLY_DISCONNECTED;
 	}
 
@@ -137,7 +161,7 @@ int CSftpConnectOpData::ParseResponse()
 int CSftpConnectOpData::Reset(int result)
 {
 	if (opState == connect_init && (result & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED) {
-		LogMessage(MessageType::Error, _("fzsftp could not be started"));
+		log(logmsg::error, _("fzsftp could not be started"));
 	}
 	if (criticalFailure) {
 		result |= FZ_REPLY_CRITICALERROR;
