@@ -4,7 +4,7 @@
 #include "delete.h"
 #include "event.h"
 #include "input_thread.h"
-#include "directorycache.h"
+#include "../directorycache.h"
 #include "directorylistingparser.h"
 #include "engineprivate.h"
 #include "file_transfer.h"
@@ -39,35 +39,15 @@ CStorjControlSocket::~CStorjControlSocket()
 
 void CStorjControlSocket::Connect(CServer const &server, Credentials const& credentials)
 {
-	LogMessage(MessageType::Status, _("Connecting to %s..."), server.Format(ServerFormat::with_optional_port));
-	SetWait(true);
-
 	currentServer_ = server;
+	credentials_ = credentials;
 
-	process_ = std::make_unique<fz::process>();
-
-	engine_.GetRateLimiter().AddObject(this);
-	Push(std::make_unique<CStorjConnectOpData>(*this, credentials));
+	Push(std::make_unique<CStorjConnectOpData>(*this));
 }
 
 void CStorjControlSocket::List(CServerPath const& path, std::wstring const& subDir, int flags)
 {
-	CServerPath newPath = currentPath_;
-	if (!path.empty()) {
-		newPath = path;
-	}
-	if (!newPath.ChangePath(subDir)) {
-		newPath.clear();
-	}
-
-	if (newPath.empty()) {
-		LogMessage(MessageType::Status, _("Retrieving directory listing..."));
-	}
-	else {
-		LogMessage(MessageType::Status, _("Retrieving directory listing of \"%s\"..."), newPath.GetPath());
-	}
-
-	Push(std::make_unique<CStorjListOpData>(*this, newPath, std::wstring(), flags, operations_.empty()));
+	Push(std::make_unique<CStorjListOpData>(*this, path, subDir, flags));
 }
 
 void CStorjControlSocket::FileTransfer(std::wstring const& localFile, CServerPath const& remotePath,
@@ -79,12 +59,12 @@ void CStorjControlSocket::FileTransfer(std::wstring const& localFile, CServerPat
 }
 
 
-void CStorjControlSocket::Delete(CServerPath const& path, std::deque<std::wstring>&& files)
+void CStorjControlSocket::Delete(CServerPath const& path, std::vector<std::wstring>&& files)
 {
 	// CFileZillaEnginePrivate should have checked this already
 	assert(!files.empty());
 
-	LogMessage(MessageType::Debug_Verbose, L"CStorjControlSocket::Delete");
+	log(logmsg::debug_verbose, L"CStorjControlSocket::Delete");
 
 	Push(std::make_unique<CStorjDeleteOpData>(*this, path, std::move(files)));
 }
@@ -94,17 +74,13 @@ void CStorjControlSocket::Resolve(CServerPath const& path, std::wstring const& f
 	Push(std::make_unique<CStorjResolveOpData>(*this, path, file, bucket, fileId, ignore_missing_file));
 }
 
-void CStorjControlSocket::Resolve(CServerPath const& path, std::deque<std::wstring> const& files, std::wstring & bucket, std::deque<std::wstring> & fileIds)
+void CStorjControlSocket::Resolve(CServerPath const& path, std::vector<std::wstring> const& files, std::wstring & bucket, std::vector<std::wstring> & fileIds)
 {
 	Push(std::make_unique<CStorjResolveManyOpData>(*this, path, files, bucket, fileIds));
 }
 
 void CStorjControlSocket::Mkdir(CServerPath const& path)
 {
-	if (operations_.empty()) {
-		LogMessage(MessageType::Status, _("Creating directory '%s'..."), path.GetPath());
-	}
-
 	auto pData = std::make_unique<CStorjMkdirOpData>(*this);
 	pData->path_ = path;
 	Push(std::move(pData));
@@ -112,7 +88,7 @@ void CStorjControlSocket::Mkdir(CServerPath const& path)
 
 void CStorjControlSocket::RemoveDir(CServerPath const& path, std::wstring const& subDir)
 {
-	LogMessage(MessageType::Debug_Verbose, L"CStorjControlSocket::RemoveDir");
+	log(logmsg::debug_verbose, L"CStorjControlSocket::RemoveDir");
 
 	auto pData = std::make_unique<CStorjRemoveDirOpData>(*this);
 	pData->path_ = path;
@@ -135,27 +111,27 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 	switch (message.type)
 	{
 	case storjEvent::Reply:
-		LogMessageRaw(MessageType::Response, message.text[0]);
+		log_raw(logmsg::reply, message.text[0]);
 		ProcessReply(FZ_REPLY_OK, message.text[0]);
 		break;
 	case storjEvent::Done:
 		ProcessReply(FZ_REPLY_OK, std::wstring());
 		break;
 	case storjEvent::Error:
-		LogMessageRaw(MessageType::Error, message.text[0]);
+		log_raw(logmsg::error, message.text[0]);
 		ProcessReply(FZ_REPLY_ERROR, message.text[0]);
 		break;
 	case storjEvent::ErrorMsg:
-		LogMessageRaw(MessageType::Error, message.text[0]);
+		log_raw(logmsg::error, message.text[0]);
 		break;
 	case storjEvent::Verbose:
-		LogMessageRaw(MessageType::Debug_Info, message.text[0]);
+		log_raw(logmsg::debug_info, message.text[0]);
 		break;
 	case storjEvent::Info:
-		LogMessageRaw(MessageType::Command, message.text[0]); // Not exactly the right message type, but it's a silent one.
+		log_raw(logmsg::command, message.text[0]); // Not exactly the right message type, but it's a silent one.
 		break;
 	case storjEvent::Status:
-		LogMessageRaw(MessageType::Status, message.text[0]);
+		log_raw(logmsg::status, message.text[0]);
 		break;
 	case storjEvent::Recv:
 		SetActive(CFileZillaEngine::recv);
@@ -165,7 +141,7 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 		break;
 	case storjEvent::Listentry:
 		if (operations_.empty() || operations_.back()->opId != Command::list) {
-			LogMessage(MessageType::Debug_Warning, L"storjEvent::Listentry outside list operation, ignoring.");
+			log(logmsg::debug_warning, L"storjEvent::Listentry outside list operation, ignoring.");
 			break;
 		}
 		else {
@@ -175,12 +151,6 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 			}
 		}
 		break;
-	case storjEvent::UsedQuotaRecv:
-		OnQuotaRequest(CRateLimiter::inbound);
-		break;
-	case storjEvent::UsedQuotaSend:
-		OnQuotaRequest(CRateLimiter::outbound);
-		break;
 	case storjEvent::Transfer:
 		{
 			auto value = fz::to_integral<int64_t>(message.text[0]);
@@ -210,7 +180,7 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 		}
 		break;
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Message type %d not handled", message.type);
+		log(logmsg::debug_warning, L"Message type %d not handled", message.type);
 		break;
 	}
 }
@@ -218,10 +188,10 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 void CStorjControlSocket::OnTerminate(std::wstring const& error)
 {
 	if (!error.empty()) {
-		LogMessageRaw(MessageType::Error, error);
+		log_raw(logmsg::error, error);
 	}
 	else {
-		LogMessageRaw(MessageType::Debug_Info, L"CStorjControlSocket::OnTerminate without error");
+		log_raw(logmsg::debug_info, L"CStorjControlSocket::OnTerminate without error");
 	}
 	if (process_) {
 		DoClose();
@@ -234,14 +204,14 @@ int CStorjControlSocket::SendCommand(std::wstring const& cmd, std::wstring const
 		SetWait(true);
 	}
 
-	LogMessageRaw(MessageType::Command, show.empty() ? cmd : show);
+	log_raw(logmsg::command, show.empty() ? cmd : show);
 
 	// Check for newlines in command
 	// a command like "ls\nrm foo/bar" is dangerous
 	if (cmd.find('\n') != std::wstring::npos ||
 		cmd.find('\r') != std::wstring::npos)
 	{
-		LogMessage(MessageType::Debug_Warning, L"Command containing newline characters, aborting.");
+		log(logmsg::debug_warning, L"Command containing newline characters, aborting.");
 		return FZ_REPLY_INTERNALERROR;
 	}
 
@@ -257,7 +227,7 @@ int CStorjControlSocket::AddToStream(std::wstring const& cmd)
 
 	std::string const str = ConvToServer(cmd, true);
 	if (str.empty()) {
-		LogMessage(MessageType::Error, _("Could not convert command to server encoding"));
+		log(logmsg::error, _("Could not convert command to server encoding"));
 		return FZ_REPLY_ERROR;
 	}
 
@@ -270,12 +240,7 @@ int CStorjControlSocket::AddToStream(std::wstring const& cmd)
 
 bool CStorjControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotification)
 {
-	if (operations_.empty() || !operations_.back()->waitForAsyncRequest) {
-		LogMessage(MessageType::Debug_Info, L"Not waiting for request reply, ignoring request reply %d", pNotification->GetRequestID());
-		return false;
-	}
-
-	operations_.back()->waitForAsyncRequest = false;
+	log(logmsg::debug_verbose, L"CStorjControlSocket::SetAsyncRequestReply");
 
 	RequestId const requestId = pNotification->GetRequestID();
 	switch(requestId)
@@ -286,7 +251,7 @@ bool CStorjControlSocket::SetAsyncRequestReply(CAsyncRequestNotification *pNotif
 			return SetFileExistsAction(pFileExistsNotification);
 		}
 	default:
-		LogMessage(MessageType::Debug_Warning, L"Unknown async request reply id: %d", requestId);
+		log(logmsg::debug_warning, L"Unknown async request reply id: %d", requestId);
 		return false;
 	}
 
@@ -301,12 +266,12 @@ void CStorjControlSocket::ProcessReply(int result, std::wstring const& reply)
 	SetWait(false);
 
 	if (operations_.empty()) {
-		LogMessage(MessageType::Debug_Info, L"Skipping reply without active operation.");
+		log(logmsg::debug_info, L"Skipping reply without active operation.");
 		return;
 	}
 
 	auto & data = *operations_.back();
-	LogMessage(MessageType::Debug_Verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
+	log(logmsg::debug_verbose, L"%s::ParseResponse() in state %d", data.name_, data.opState);
 	int res = data.ParseResponse();
 	if (res == FZ_REPLY_OK) {
 		ResetOperation(FZ_REPLY_OK);
@@ -332,13 +297,13 @@ int CStorjControlSocket::ResetOperation(int nErrorCode)
 	if (!operations_.empty() && operations_.back()->opId == Command::connect) {
 		auto &data = static_cast<CStorjConnectOpData &>(*operations_.back());
 		if (data.opState == connect_init && nErrorCode & FZ_REPLY_ERROR && (nErrorCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED) {
-			LogMessage(MessageType::Error, _("fzstorj could not be started"));
+			log(logmsg::error, _("fzstorj could not be started"));
 		}
 	}
 	if (!operations_.empty() && operations_.back()->opId == Command::del && !(nErrorCode & FZ_REPLY_DISCONNECTED)) {
 		auto &data = static_cast<CStorjDeleteOpData &>(*operations_.back());
 		if (data.needSendListing_) {
-			SendDirectoryListingNotification(data.path_, false, false);
+			SendDirectoryListingNotification(data.path_, false);
 		}
 	}
 
@@ -347,8 +312,6 @@ int CStorjControlSocket::ResetOperation(int nErrorCode)
 
 int CStorjControlSocket::DoClose(int nErrorCode)
 {
-	engine_.GetRateLimiter().RemoveObject(this);
-
 	if (process_) {
 		process_->kill();
 	}
@@ -379,33 +342,6 @@ void CStorjControlSocket::Cancel()
 	}
 }
 
-void CStorjControlSocket::OnRateAvailable(CRateLimiter::rate_direction direction)
-{
-	//OnQuotaRequest(direction);
-}
-
-void CStorjControlSocket::OnQuotaRequest(CRateLimiter::rate_direction direction)
-{
-	/*int64_t bytes = GetAvailableBytes(direction);
-	if (bytes > 0) {
-		int b;
-		if (bytes > INT_MAX) {
-			b = INT_MAX;
-		}
-		else {
-			b = bytes;
-		}
-		AddToStream(fz::sprintf(L"-%d%d,%d\n", direction, b, engine_.GetOptions().GetOptionVal(OPTION_SPEEDLIMIT_INBOUND + static_cast<int>(direction))));
-		UpdateUsage(direction, b);
-	}
-	else if (bytes == 0) {
-		Wait(direction);
-	}
-	else if (bytes < 0) {
-		AddToStream(fz::sprintf(L"-%d-\n", direction));
-	}*/
-}
-
 void CStorjControlSocket::operator()(fz::event_base const& ev)
 {
 	if (fz::dispatch<CStorjEvent, StorjTerminateEvent>(ev, this,
@@ -421,3 +357,15 @@ std::wstring CStorjControlSocket::QuoteFilename(std::wstring const& filename)
 {
 	return L"\"" + fz::replaced_substrings(filename, L"\"", L"\"\"") + L"\"";
 }
+
+void CStorjControlSocket::Push(std::unique_ptr<COpData> && pNewOpData)
+{
+	CControlSocket::Push(std::move(pNewOpData));
+	if (operations_.size() == 1 && operations_.back()->opId != Command::connect) {
+		if (!process_) {
+			std::unique_ptr<COpData> connOp = std::make_unique<CStorjConnectOpData>(*this);
+			connOp->topLevelOperation_ = true;
+			CControlSocket::Push(std::move(connOp));
+		}
+	}
+}
