@@ -3,6 +3,7 @@
 
 #include <map>
 #include <memory>
+#include <vector>
 
 enum ServerProtocol
 {
@@ -102,6 +103,15 @@ enum class ProtocolFeature
 	Security // Encryption, integrity protection and authentication
 };
 
+enum class CaseSensitivity
+{
+	unspecified,
+	yes,
+	no
+};
+
+CaseSensitivity GetCaseSensitivity(ServerProtocol protocol);
+
 class Credentials;
 class CServerPath;
 class CServer final
@@ -112,6 +122,12 @@ public:
 	CServer() = default;
 	CServer(ServerProtocol protocol, ServerType type, std::wstring const& host, unsigned int);
 
+	CServer(CServer const&) = default;
+	CServer& operator=(CServer const&) = default;
+
+	CServer(CServer &&) noexcept = default;
+	CServer& operator=(CServer &&) noexcept = default;
+
 	void clear();
 
 	void SetType(ServerType type);
@@ -131,7 +147,6 @@ public:
 
 	void SetUser(std::wstring const& user);
 
-	CServer& operator=(const CServer &op);
 	bool operator==(const CServer &op) const;
 	bool operator<(const CServer &op) const;
 	bool operator!=(const CServer &op) const;
@@ -152,7 +167,6 @@ public:
 	std::wstring Format(ServerFormat formatType, Credentials const& credentials) const;
 
 	bool SetEncodingType(CharsetEncoding type, std::wstring const& encoding = std::wstring());
-	bool SetCustomEncoding(std::wstring const& encoding);
 	CharsetEncoding GetEncodingType() const;
 	std::wstring GetCustomEncoding() const;
 
@@ -169,17 +183,14 @@ public:
 	bool HasFeature(ProtocolFeature const feature) const;
 	static bool ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature const feature);
 
+	CaseSensitivity GetCaseSensitivity() const;
+
 	// These commands will be executed after a successful login.
 	std::vector<std::wstring> const& GetPostLoginCommands() const { return m_postLoginCommands; }
 	bool SetPostLoginCommands(std::vector<std::wstring> const& postLoginCommands);
 
 	void SetBypassProxy(bool val);
 
-	// Abstract server name.
-	// Not compared in ==, < and related operators
-	void SetName(std::wstring const& name) { m_name = name; }
-	std::wstring GetName() const { return m_name; }
-
 	static std::wstring GetNameFromServerType(ServerType type);
 	static ServerType GetServerTypeFromName(std::wstring const& name);
 
@@ -187,10 +198,11 @@ public:
 	explicit operator bool() const { return !empty(); }
 
 	void ClearExtraParameters();
-	std::wstring GetExtraParameter(std::string const& name) const;
-	std::map<std::string, std::wstring> const& GetExtraParameters() const;
-	void SetExtraParameter(std::string const& name, std::wstring const& value);
-	void ClearExtraParameter(std::string const& name);
+	std::wstring GetExtraParameter(std::string_view const& name) const;
+	std::map<std::string, std::wstring, std::less<>> const& GetExtraParameters() const;
+	bool HasExtraParameter(std::string_view const& name) const;
+	void SetExtraParameter(std::string_view const& name, std::wstring const& value);
+	void ClearExtraParameter(std::string_view const& name);
 
 protected:
 	ServerProtocol m_protocol{UNKNOWN};
@@ -203,12 +215,11 @@ protected:
 	int m_maximumMultipleConnections{};
 	CharsetEncoding m_encodingType{ENCODING_AUTO};
 	std::wstring m_customEncoding;
-	std::wstring m_name;
 
 	std::vector<std::wstring> m_postLoginCommands;
 	bool m_bypassProxy{};
 
-	std::map<std::string, std::wstring> extraParameters_;
+	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
 
 
@@ -283,13 +294,14 @@ public:
 	std::wstring keyFile_;
 
 	void ClearExtraParameters();
-	std::wstring GetExtraParameter(std::string const& name) const;
-	std::map<std::string, std::wstring> const& GetExtraParameters() const;
-	void SetExtraParameter(ServerProtocol protocol, std::string const& name, std::wstring const& value);
+	std::wstring GetExtraParameter(std::string_view const& name) const;
+	std::map<std::string, std::wstring, std::less<>> const& GetExtraParameters() const;
+	bool HasExtraParameter(std::string_view const& name) const;
+	void SetExtraParameter(ServerProtocol protocol, std::string_view const& name, std::wstring const& value);
 
 protected:
 	std::wstring password_;
-	std::map<std::string, std::wstring> extraParameters_;
+	std::map<std::string, std::wstring, std::less<>> extraParameters_;
 };
 
 struct ServerHandleData {
