@@ -8,6 +8,7 @@
 #include "filezillaapp.h"
 #include "filter.h"
 #include "file_utils.h"
+#include "infotext.h"
 #include "inputdialog.h"
 #include <algorithm>
 #include "dndobjects.h"
@@ -256,6 +257,7 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_DELETE"), CLocalListView::OnMenuDelete)
 	EVT_MENU(XRCID("ID_RENAME"), CLocalListView::OnMenuRename)
 	EVT_KEY_DOWN(CLocalListView::OnKeyDown)
+	EVT_SIZE(CLocalListView::OnSize)
 	EVT_LIST_BEGIN_DRAG(wxID_ANY, CLocalListView::OnBeginDrag)
 	EVT_MENU(XRCID("ID_OPEN"), CLocalListView::OnMenuOpen)
 	EVT_MENU(XRCID("ID_EDIT"), CLocalListView::OnMenuEdit)
@@ -278,7 +280,7 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 	m_state.RegisterHandler(this, STATECHANGE_LOCAL_REFRESH_FILE);
 	m_state.RegisterHandler(this, STATECHANGE_SERVER);
 
-	const unsigned long widths[4] = { 120, 80, 100, 120 };
+	const unsigned long widths[4] = { 170, 80, 120, 120 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
@@ -297,6 +299,8 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 	EnablePrefixSearch(true);
 
 	m_windowTinter = std::make_unique<CWindowTinter>(*GetMainWindow());
+
+	m_pInfoText = new CInfoText(*this);
 }
 
 CLocalListView::~CLocalListView()
@@ -311,7 +315,6 @@ CLocalListView::~CLocalListView()
 
 bool CLocalListView::DisplayDir(CLocalPath const& dirname)
 {
-
 	CancelLabelEdit();
 
 	std::wstring focused;
@@ -384,11 +387,26 @@ regular_dir:
 		CStateFilterManager const& filter = m_state.GetStateFilterManager();
 		fz::local_filesys local_filesys;
 
-		if (!local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false)) {
+		auto result = local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false);
+		if (!result) {
+			
+			if (result.error_ == fz::result::noperm) {
+				SetInfoText(_("You do not have permission to list this directory"));
+			}
+			else {
+				SetInfoText(_("Could not list directory contents"));
+			}
+
 			SetItemCount(1);
+			if (m_pFilelistStatusBar) {
+				m_pFilelistStatusBar->SetDirectoryContents(0, 0, 0, 0, 0);
+			}
+
 			return false;
 		}
 
+		SetInfoText(wxString());
+
 		int64_t totalSize{};
 		int unknown_sizes = 0;
 		int totalFileCount = 0;
@@ -409,23 +427,23 @@ regular_dir:
 			m_fileData.push_back(data);
 			if (!filter.FilenameFiltered(data.name, m_dir.GetPath(), data.dir, data.size, true, data.attributes, data.time)) {
 				if (data.dir) {
-					totalDirCount++;
+					++totalDirCount;
 				}
 				else {
 					if (data.size != -1) {
 						totalSize += data.size;
 					}
 					else {
-						unknown_sizes++;
+						++unknown_sizes;
 					}
-					totalFileCount++;
+					++totalFileCount;
 				}
 				m_indexMapping.push_back(num);
 			}
 			else {
-				hidden++;
+				++hidden;
 			}
-			num++;
+			++num;
 		}
 
 		if (m_pFilelistStatusBar) {
@@ -623,6 +641,7 @@ void CLocalListView::OnMenuEnter(wxCommandEvent &)
 #ifdef __WXMSW__
 void CLocalListView::DisplayDrives()
 {
+	SetInfoText(wxString());
 	int count = m_fileData.size();
 
 	std::vector<std::wstring> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
@@ -658,6 +677,8 @@ void CLocalListView::DisplayDrives()
 
 void CLocalListView::DisplayShares(wxString computer)
 {
+	SetInfoText(wxString());
+
 	// Cast through a union to avoid warning about breaking strict aliasing rule
 	union
 	{
@@ -1362,6 +1383,14 @@ void CLocalListView::ReselectItems(const std::vector<std::wstring>& selectedName
 	}
 }
 
+void CLocalListView::OnSize(wxSizeEvent& event)
+{
+	event.Skip();
+	if (m_pInfoText) {
+		m_pInfoText->Reposition();
+	}
+}
+
 void CLocalListView::OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR) {
@@ -1371,7 +1400,12 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, std
 		ApplyCurrentFilter();
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		if (m_windowTinter) {
+			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
+		if (m_pInfoText) {
+			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_LOCAL_REFRESH_FILE);
@@ -1379,6 +1413,22 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, std
 	}
 }
 
+void CLocalListView::SetInfoText(wxString const& text)
+{
+	if (!m_pInfoText) {
+		return;
+	}
+
+	if (IsComparing() || text.empty()) {
+		m_pInfoText->Hide();
+	}
+	else {
+		m_pInfoText->SetText(text);
+		m_pInfoText->Reposition();
+		m_pInfoText->Show();
+	}
+}
+
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
 	if (COptions::Get()->GetOptionVal(OPTION_DND_DISABLED) != 0) {
@@ -1660,7 +1710,7 @@ void CLocalListView::StartComparison()
 	}
 }
 
-bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
+bool CLocalListView::get_next_file(std::wstring_view & name, std::wstring & path, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
@@ -1683,6 +1733,8 @@ bool CLocalListView::get_next_file(std::wstring & name, bool& dir, int64_t& size
 
 void CLocalListView::FinishComparison()
 {
+	SetInfoText(wxString());
+
 	SetItemCount(m_indexMapping.size());
 
 	ComparisonRestoreSelections();
@@ -1690,11 +1742,9 @@ void CLocalListView::FinishComparison()
 	RefreshListOnly();
 
 	CComparableListing* pOther = GetOther();
-	if (!pOther) {
-		return;
+	if (pOther) {
+		pOther->ScrollTopItem(GetTopItem());
 	}
-
-	pOther->ScrollTopItem(GetTopItem());
 }
 
 bool CLocalListView::CanStartComparison()
