@@ -1,5 +1,5 @@
-#ifndef __OPTIONS_H__
-#define __OPTIONS_H__
+#ifndef FILEZILLA_INTERFACE_OPTIONS_HEADER
+#define FILEZILLA_INTERFACE_OPTIONS_HEADER
 
 #include "local_path.h"
 
@@ -19,7 +19,6 @@ enum interfaceOptions
 	OPTION_ASCIINOEXT,
 	OPTION_ASCIIDOTFILE,
 	OPTION_LANGUAGE,
-	OPTION_LASTSERVERPATH,
 	OPTION_CONCURRENTDOWNLOADLIMIT,
 	OPTION_CONCURRENTUPLOADLIMIT,
 	OPTION_UPDATECHECK,
@@ -39,7 +38,6 @@ enum interfaceOptions
 	OPTION_SHOW_TREE_REMOTE,
 	OPTION_FILEPANE_LAYOUT,
 	OPTION_FILEPANE_SWAP,
-	OPTION_LASTLOCALDIR,
 	OPTION_FILELIST_DIRSORT,
 	OPTION_FILELIST_NAMESORT,
 	OPTION_QUEUE_SUCCESSFUL_AUTOCLEAR,
@@ -73,7 +71,6 @@ enum interfaceOptions
 	OPTION_FILTERTOGGLESTATE,
 	OPTION_SHOW_QUICKCONNECT,
 	OPTION_MESSAGELOG_POSITION,
-	OPTION_LAST_CONNECTED_SITE,
 	OPTION_DOUBLECLICK_ACTION_FILE,
 	OPTION_DOUBLECLICK_ACTION_DIRECTORY,
 	OPTION_MINIMIZE_TRAY,
@@ -93,13 +90,15 @@ enum interfaceOptions
 	OPTION_SPEED_DISPLAY,
 	OPTION_TOOLBAR_HIDDEN,
 	OPTION_STRIP_VMS_REVISION,
-	OPTION_INTERFACE_SITEMANAGER_ON_STARTUP,
+	OPTION_STARTUP_ACTION,
 	OPTION_PROMPTPASSWORDSAVE,
 	OPTION_PERSISTENT_CHOICES,
 	OPTION_QUEUE_COMPLETION_ACTION,
 	OPTION_QUEUE_COMPLETION_COMMAND,
 	OPTION_DND_DISABLED,
 	OPTION_DISABLE_UPDATE_FOOTER,
+	OPTION_MASTERPASSWORDENCRYPTOR,
+	OPTION_TAB_DATA,
 
 	// Default/internal options
 	OPTION_DEFAULT_SETTINGSDIR, // guaranteed to be (back)slash-terminated
@@ -113,37 +112,41 @@ enum interfaceOptions
 
 struct t_OptionsCache
 {
-	bool operator==(std::wstring const& v) const { return strValue == v; }
+	bool operator==(std::wstring_view const& v) const { return strValue == v; }
 	bool operator==(int v) const { return numValue == v; }
-	t_OptionsCache& operator=(std::wstring const& v);
+	bool operator==(std::unique_ptr<pugi::xml_document> const& v) const { return *xmlValue == *v; }
+	t_OptionsCache& operator=(std::wstring_view const& v);
+	t_OptionsCache& operator=(std::wstring && v);
 	t_OptionsCache& operator=(int v);
+	t_OptionsCache& operator=(std::unique_ptr<pugi::xml_document> const& v);
 
 	bool from_default;
 	int numValue;
 	std::wstring strValue;
+	std::unique_ptr<pugi::xml_document> xmlValue;
 };
 
 class CXmlFile;
-class COptions : public wxEvtHandler, public COptionsBase
+class COptions final : public wxEvtHandler, public COptionsBase
 {
 public:
 	virtual int GetOptionVal(unsigned int nID);
 	virtual std::wstring GetOption(unsigned int nID);
+	virtual std::unique_ptr<pugi::xml_document> GetOptionXml(unsigned int nID);
 
 	virtual bool SetOption(unsigned int nID, int value);
-	virtual bool SetOption(unsigned int nID, std::wstring const& value);
+	virtual bool SetOption(unsigned int nID, std::wstring_view const& value);
+	virtual bool SetOptionXml(unsigned int nID, pugi::xml_node const& value);
 
 	bool OptionFromFzDefaultsXml(unsigned int nID);
 
-	void SetLastServer(const CServer& server);
-	bool GetLastServer(CServer& server);
-
 	static COptions* Get();
 	static void Init();
 	static void Destroy();
 
 	void Import(pugi::xml_node element);
 
+	void RequireCleanup();
 	void SaveIfNeeded();
 
 	static CLocalPath GetUnadjustedSettingsDir();
@@ -154,15 +157,13 @@ protected:
 	virtual ~COptions();
 
 	int Validate(unsigned int nID, int value);
-	std::wstring Validate(unsigned int nID, std::wstring const& value);
+	std::wstring Validate(unsigned int nID, std::wstring_view const& value);
+	std::unique_ptr<pugi::xml_document> Validate(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value);
 
 	template<typename T> void ContinueSetOption(unsigned int nID, T const& value);
 	void SetXmlValue(unsigned int nID, int value);
-	void SetXmlValue(unsigned int nID, std::wstring const& value);
-
-	// path is element path below document root, separated by slashes
-	void SetServer(std::wstring path, const CServer& server);
-	bool GetServer(std::wstring path, CServer& server);
+	void SetXmlValue(unsigned int nID, std::wstring_view const& value);
+	void SetXmlValue(unsigned int nID, std::unique_ptr<pugi::xml_document> const& value);
 
 	pugi::xml_node CreateSettingsXmlElement();
 
@@ -173,19 +174,19 @@ protected:
 	CLocalPath InitSettingsDir();
 	void SetDefaultValues();
 
+	bool Cleanup(); // Removes all unknown elements from the XML
 	void Save();
 
 	void NotifyChangedOptions();
 
-	CXmlFile* m_pXmlFile;
+	std::unique_ptr<CXmlFile> xmlFile_;
 
 	t_OptionsCache m_optionsCache[OPTIONS_NUM];
 
-	CServer* m_pLastServer;
-
 	static COptions* m_theOptions;
 
 	wxTimer m_save_timer;
+	bool needsCleanup_{};
 
 	DECLARE_EVENT_TABLE()
 	void OnTimer(wxTimerEvent& event);
@@ -195,4 +196,4 @@ protected:
 	changed_options_t changedOptions_;
 };
 
-#endif //__OPTIONS_H__
+#endif
