@@ -758,12 +758,6 @@ bool CQueueView::TryStartNextTransfer()
 
 void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification const& notification)
 {
-	if (notification.nReplyCode & FZ_REPLY_DISCONNECTED &&
-		notification.commandId == ::Command::none)
-	{
-		// Queue is not interested in disconnect notifications
-		return;
-	}
 	wxASSERT(notification.commandId != ::Command::none);
 
 	// Cancel pending requests
@@ -791,6 +785,9 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 		else {
 			reason = ResetReason::reset;
 		}
+		if (pEngineData->state == t_EngineData::connect) {
+			SetActive(false);
+		}
 		ResetEngine(*pEngineData, reason);
 		return;
 	}
@@ -853,10 +850,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 					return;
 				}
 			}
-
-			if (!pEngineData->transient) {
-				SwitchEngine(&pEngineData);
-			}
 		}
 		break;
 	case t_EngineData::transfer:
@@ -911,11 +904,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 				return;
 			}
 		}
-		if (replyCode & FZ_REPLY_DISCONNECTED) {
-			if (!SwitchEngine(&pEngineData)) {
-				pEngineData->state = t_EngineData::connect;
-			}
-		}
 		break;
 	case t_EngineData::mkdir:
 		if (replyCode == FZ_REPLY_OK) {
@@ -926,15 +914,6 @@ void CQueueView::ProcessReply(t_EngineData* pEngineData, COperationNotification
 			if (!IncreaseErrorCount(*pEngineData)) {
 				return;
 			}
-
-			if (pEngineData->transient) {
-				ResetEngine(*pEngineData, ResetReason::retry);
-				return;
-			}
-
-			if (!SwitchEngine(&pEngineData)) {
-				pEngineData->state = t_EngineData::connect;
-			}
 		}
 		else {
 			// Cannot retry
@@ -2556,6 +2535,9 @@ void CQueueView::OnTimer(wxTimerEvent& event)
 void CQueueView::DeleteEngines()
 {
 	for (auto & engineData : m_engineData) {
+		if (m_pAsyncRequestQueue) {
+			m_pAsyncRequestQueue->ClearPending(engineData->pEngine);
+		}
 		delete engineData;
 	}
 	m_engineData.clear();
@@ -2565,38 +2547,48 @@ void CQueueView::OnSetPriority(wxCommandEvent& event)
 {
 #ifndef __WXMSW__
 	// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-	if (!GetSelectedItemCount())
+	if (!GetSelectedItemCount()) {
 		return;
+	}
 #endif
 
 	QueuePriority priority;
 
 	const int id = event.GetId();
-	if (id == XRCID("ID_PRIORITY_LOWEST"))
+	if (id == XRCID("ID_PRIORITY_LOWEST")) {
 		priority = QueuePriority::lowest;
-	else if (id == XRCID("ID_PRIORITY_LOW"))
+	}
+	else if (id == XRCID("ID_PRIORITY_LOW")) {
 		priority = QueuePriority::low;
-	else if (id == XRCID("ID_PRIORITY_HIGH"))
+	}
+	else if (id == XRCID("ID_PRIORITY_HIGH")) {
 		priority = QueuePriority::high;
-	else if (id == XRCID("ID_PRIORITY_HIGHEST"))
+	}
+	else if (id == XRCID("ID_PRIORITY_HIGHEST")) {
 		priority = QueuePriority::highest;
-	else
+	}
+	else {
 		priority = QueuePriority::normal;
+	}
 
 
 	CQueueItem* pSkip = 0;
 	long item = -1;
 	while (-1 != (item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED))) {
 		CQueueItem* pItem = GetQueueItem(item);
-		if (!pItem)
+		if (!pItem) {
 			continue;
+		}
 
-		if (pItem->GetType() == QueueItemType::Server)
+		if (pItem->GetType() == QueueItemType::Server) {
 			pSkip = pItem;
-		else if (pItem->GetTopLevelItem() == pSkip)
+		}
+		else if (pItem->GetTopLevelItem() == pSkip) {
 			continue;
-		else
+		}
+		else {
 			pSkip = 0;
+		}
 
 		pItem->SetPriority(priority);
 	}
@@ -2871,72 +2863,6 @@ void CQueueView::OnActionAfterTimerTick()
 }
 #endif
 
-bool CQueueView::SwitchEngine(t_EngineData** ppEngineData)
-{
-	if (m_engineData.size() < 2) {
-		return false;
-	}
-
-	t_EngineData* pEngineData = *ppEngineData;
-	for (auto & pNewEngineData : m_engineData) {
-		if (pNewEngineData == pEngineData) {
-			continue;
-		}
-
-		if (pNewEngineData->active || pNewEngineData->transient) {
-			continue;
-		}
-
-		if (pNewEngineData->lastSite != pEngineData->lastSite) {
-			continue;
-		}
-
-		if (!pNewEngineData->pEngine->IsConnected()) {
-			continue;
-		}
-
-		wxASSERT(!pNewEngineData->pItem);
-		pNewEngineData->pItem = pEngineData->pItem;
-		pNewEngineData->pItem->m_pEngineData = pNewEngineData;
-		pEngineData->pItem = 0;
-
-		pNewEngineData->active = true;
-		pEngineData->active = false;
-
-		delete pNewEngineData->m_idleDisconnectTimer;
-		pNewEngineData->m_idleDisconnectTimer = 0;
-
-		// Swap status line
-		CStatusLineCtrl* pOldStatusLineCtrl = pNewEngineData->pStatusLineCtrl;
-		pNewEngineData->pStatusLineCtrl = pEngineData->pStatusLineCtrl;
-		if (pNewEngineData->pStatusLineCtrl) {
-			pNewEngineData->pStatusLineCtrl->SetEngineData(pNewEngineData);
-		}
-		if (pOldStatusLineCtrl) {
-			pEngineData->pStatusLineCtrl = pOldStatusLineCtrl;
-			pEngineData->pStatusLineCtrl->SetEngineData(pEngineData);
-		}
-
-		// Set new state
-		if (pNewEngineData->pItem->GetType() == QueueItemType::File) {
-			pNewEngineData->state = t_EngineData::transfer;
-		}
-		else {
-			pNewEngineData->state = t_EngineData::mkdir;
-		}
-		if (pNewEngineData->pStatusLineCtrl) {
-			pNewEngineData->pStatusLineCtrl->ClearTransferStatus();
-		}
-
-		pEngineData->state = t_EngineData::none;
-
-		*ppEngineData = pNewEngineData;
-		return true;
-	}
-
-	return false;
-}
-
 bool CQueueView::IsOtherEngineConnected(t_EngineData* pEngineData)
 {
 	for (auto const* current : m_engineData) {
