@@ -1,5 +1,5 @@
-#ifndef __QUEUEVIEW_H__
-#define __QUEUEVIEW_H__
+#ifndef FILEZILLA_INTERFACE_QUEUEVIEW_HEADER
+#define FILEZILLA_INTERFACE_QUEUEVIEW_HEADER
 
 #include "dndobjects.h"
 #include "queue.h"
@@ -28,6 +28,7 @@ enum type {
 	Reboot,
 	Shutdown,
 	Sleep,
+	CloseOnce,
 
 	Count
 };
@@ -74,7 +75,7 @@ struct t_EngineData final
 	} state;
 
 	CFileItem* pItem;
-	CServer lastServer;
+	Site lastSite;
 	CStatusLineCtrl* pStatusLineCtrl;
 	wxTimer* m_idleDisconnectTimer;
 };
@@ -105,7 +106,7 @@ private:
 	bool trigger_{};
 };
 
-class CQueueView : public CQueueViewBase, public COptionChangeEventHandler, private EngineNotificationHandler
+class CQueueView final : public CQueueViewBase, public COptionChangeEventHandler, public CGlobalStateEventHandler, private EngineNotificationHandler
 {
 	friend class CQueueViewDropTarget;
 	friend class CQueueViewFailed;
@@ -118,12 +119,12 @@ public:
 	bool QueueFile(const bool queueOnly, const bool download,
 		std::wstring const& localFile, std::wstring const& remoteFile,
 		const CLocalPath& localPath, const CServerPath& remotePath,
-		const CServer& server, int64_t size, CEditHandler::fileType edit = CEditHandler::none,
+		Site const& site, int64_t size, CEditHandler::fileType edit = CEditHandler::none,
 		QueuePriority priority = QueuePriority::normal);
 
 	void QueueFile_Finish(const bool start); // Need to be called after QueueFile
 	bool QueueFiles(const bool queueOnly, CLocalPath const& localPath, const CRemoteDataObject& dataObject);
-	bool QueueFiles(const bool queueOnly, CServer const& server, CLocalRecursiveOperation::listing const& listing);
+	bool QueueFiles(const bool queueOnly, Site const& site, CLocalRecursiveOperation::listing const& listing);
 
 	bool empty() const;
 	int IsActive() const { return m_activeMode; }
@@ -141,9 +142,9 @@ public:
 	void LoadQueueFromXML();
 	void ImportQueue(pugi::xml_node element, bool updateSelections);
 
-	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem);
+	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem) override;
 
-	virtual void CommitChanges();
+	virtual void CommitChanges() override;
 
 	void ProcessNotification(CFileZillaEngine* pEngine, std::unique_ptr<CNotification>&& pNotification);
 
@@ -167,7 +168,7 @@ protected:
 	WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
 #endif
 
-	virtual void OnOptionsChanged(changed_options_t const& options);
+	virtual void OnOptionsChanged(changed_options_t const& options) override;
 
 	void AdvanceQueue(bool refresh = true);
 	bool TryStartNextTransfer();
@@ -179,7 +180,7 @@ protected:
 	void ProcessReply(t_EngineData* pEngineData, COperationNotification const& notification);
 	void SendNextCommand(t_EngineData& engineData);
 
-	enum ResetReason
+	enum class ResetReason
 	{
 		success,
 		failure,
@@ -191,7 +192,7 @@ protected:
 	void ResetEngine(t_EngineData& data, const ResetReason reason);
 	void DeleteEngines();
 
-	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true);
+	virtual bool RemoveItem(CQueueItem* item, bool destroy, bool updateItemCount = true, bool updateSelections = true, bool forward = true) override;
 
 	// Stops processing of given item
 	// Returns true on success, false if it would block
@@ -203,7 +204,7 @@ protected:
 	void UpdateStatusLinePositions();
 	void CalculateQueueSize();
 	void DisplayQueueSize();
-	void SaveQueue();
+	void SaveQueue(bool silent = false);
 
 	bool IsActionAfter(ActionAfterState::type);
 	void ActionAfter(bool warned = false);
@@ -221,14 +222,9 @@ protected:
 	CServerPath m_last_refresh_path;
 	fz::monotonic_clock m_last_refresh_listing_time;
 
-	// Called from Process Reply.
-	// After a disconnect, check if there's another idle engine that
-	// is already connected.
-	bool SwitchEngine(t_EngineData** ppEngineData);
-
 	bool IsOtherEngineConnected(t_EngineData* pEngineData);
 
-	t_EngineData* GetIdleEngine(const CServer* pServer = 0, bool allowTransient = false);
+	t_EngineData* GetIdleEngine(Site const& site = Site(), bool allowTransient = false);
 	t_EngineData* GetEngineData(const CFileZillaEngine* pEngine);
 
 	std::vector<t_EngineData*> m_engineData;
@@ -266,7 +262,7 @@ protected:
 
 	std::list<CFileZillaEngine*> m_waitingForPassword;
 
-	virtual void OnPostScroll();
+	virtual void OnPostScroll() override;
 
 	int GetLineHeight();
 	int m_line_height;
@@ -290,13 +286,15 @@ protected:
 	// Unit is byte/s.
 	wxFileOffset GetCurrentSpeed(bool countDownload, bool countUpload);
 
-	virtual void OnEngineEvent(CFileZillaEngine* engine);
+	virtual void OnEngineEvent(CFileZillaEngine* engine) override;
 	void DoOnEngineEvent(CFileZillaEngine* engine);
 
 	void OnAskPassword();
 
 	std::weak_ptr<CActionAfterBlocker> m_actionAfterBlocker;
 
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const& data, const void* data2) override;
+
 	DECLARE_EVENT_TABLE()
 	void OnChar(wxKeyEvent& event);
 
@@ -317,6 +315,8 @@ protected:
 	void OnActionAfterTimerTick();
 
 	void OnSize(wxSizeEvent& event);
+
+	void OnColumnClicked(wxListEvent &event);
 };
 
 #endif
