@@ -12,6 +12,7 @@
 #include "filezillaapp.h"
 #include "filter.h"
 #include "graphics.h"
+#include "infotext.h"
 #include "inputdialog.h"
 #include "Options.h"
 #include "queue.h"
@@ -21,7 +22,6 @@
 #include "timeformatting.h"
 
 #include <wx/clipbrd.h>
-#include <wx/dcclient.h>
 
 #include <algorithm>
 
@@ -293,67 +293,6 @@ protected:
 	CRemoteListView *m_pRemoteListView{};
 };
 
-class CInfoText final : public wxWindow
-{
-public:
-	CInfoText(wxWindow* parent)
-		: m_tinter(*this)
-	{
-		Hide();
-		Create(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize);
-
-		SetForegroundColour(parent->GetForegroundColour());
-		SetBackgroundColour(parent->GetBackgroundColour());
-		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
-
-#ifdef __WXMSW__
-		if (GetLayoutDirection() != wxLayout_RightToLeft)
-			SetDoubleBuffered(true);
-#endif
-	}
-
-	void SetText(wxString const& text)
-	{
-		if (text == m_text) {
-			return;
-		}
-
-		m_text = text;
-		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
-	}
-
-	wxSize GetTextSize() const { return m_textSize; }
-
-	bool AcceptsFocus() const { return false; }
-
-	void SetBackgroundTint(wxColour const& colour) {
-		m_tinter.SetBackgroundTint(colour);
-	}
-
-protected:
-	wxString m_text;
-
-	void OnPaint(wxPaintEvent&)
-	{
-		wxPaintDC paintDc(this);
-
-		paintDc.SetFont(GetFont());
-		paintDc.SetTextForeground(GetForegroundColour());
-
-		paintDc.DrawText(m_text, 0, 0);
-	};
-
-	wxSize m_textSize;
-
-	CWindowTinter m_tinter;
-
-	DECLARE_EVENT_TABLE()
-};
-
-BEGIN_EVENT_TABLE(CInfoText, wxWindow)
-EVT_PAINT(CInfoText::OnPaint)
-END_EVENT_TABLE()
-
 BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_LIST_ITEM_ACTIVATED(wxID_ANY, CRemoteListView::OnItemActivated)
 	EVT_CONTEXT_MENU(CRemoteListView::OnContextMenu)
@@ -388,11 +327,11 @@ CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQue
 
 	m_dropTarget = -1;
 
-	m_pInfoText = new CInfoText(this);
+	m_pInfoText = new CInfoText(*this);
 
-	m_pDirectoryListing = 0;
+	m_pDirectoryListing = nullptr;
 
-	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
+	const unsigned long widths[6] = { 150, 75, 80, 100, 80, 85 };
 
 	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
@@ -409,7 +348,7 @@ CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQue
 
 	InitSort(OPTION_REMOTEFILELIST_SORTORDER);
 
-	SetDirectoryListing(0);
+	SetDirectoryListing(nullptr);
 
 	SetDropTarget(new CRemoteListViewDropTarget(this));
 
@@ -1513,7 +1452,8 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 
 	CFilterManager filter;
 	if (CServer::ProtocolHasFeature(m_state.GetSite().server.GetProtocol(), ProtocolFeature::RecursiveDelete) && !filter.HasActiveRemoteFilters()) {
-		std::deque<std::wstring> filesToDelete;
+		std::vector<std::wstring> filesToDelete;
+		filesToDelete.reserve(count_files);
 
 		for (item = -1; ;) {
 			item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
@@ -1551,7 +1491,8 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 		CRemoteRecursiveOperation* pRecursiveOperation = m_state.GetRemoteRecursiveOperation();
 		wxASSERT(pRecursiveOperation);
 
-		std::deque<std::wstring> filesToDelete;
+		std::vector<std::wstring> filesToDelete;
+		filesToDelete.reserve(count_files);
 
 		recursion_root root(m_pDirectoryListing->path, false);
 		for (item = -1; ;) {
@@ -2164,40 +2105,9 @@ void CRemoteListView::ReselectItems(std::vector<std::wstring>& selectedNames, st
 void CRemoteListView::OnSize(wxSizeEvent& event)
 {
 	event.Skip();
-	RepositionInfoText();
-}
-
-void CRemoteListView::RepositionInfoText()
-{
-	if (!m_pInfoText) {
-		return;
+	if (m_pInfoText) {
+		m_pInfoText->Reposition();
 	}
-
-	wxRect rect = GetClientRect();
-	wxSize size = m_pInfoText->GetTextSize();
-
-	if (!GetItemCount()) {
-		rect.y = 60;
-	}
-	else {
-		wxRect itemRect;
-		GetItemRect(0, itemRect);
-		rect.y = wxMax(60, itemRect.GetBottom() + 1);
-	}
-	rect.x = rect.x + (rect.width - size.x) / 2;
-	rect.width = size.x;
-	rect.height = size.y;
-
-	m_pInfoText->SetSize(rect);
-#ifdef __WXMSW__
-	if (GetLayoutDirection() != wxLayout_RightToLeft) {
-		m_pInfoText->Refresh(true);
-		m_pInfoText->Update();
-	}
-	else
-#endif
-		m_pInfoText->Refresh(false);
-
 }
 
 void CRemoteListView::OnStateChange(t_statechange_notifications notification, std::wstring const& data, const void* data2)
@@ -2210,8 +2120,12 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, st
 		LinkIsNotDir(*(CServerPath*)data2, data);
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
-		m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+		if (m_windowTinter) {
+			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
+		if (m_pInfoText) {
+			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+		}
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -2221,6 +2135,10 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, st
 
 void CRemoteListView::SetInfoText()
 {
+	if (!m_pInfoText) {
+		return;
+	}
+
 	wxString text;
 	if (!IsComparing()) {
 		if (!m_pDirectoryListing) {
@@ -2240,7 +2158,7 @@ void CRemoteListView::SetInfoText()
 	}
 	else {
 		m_pInfoText->SetText(text);
-		RepositionInfoText();
+		m_pInfoText->Reposition();
 		m_pInfoText->Show();
 	}
 }
@@ -2513,7 +2431,7 @@ void CRemoteListView::StartComparison()
 	}
 }
 
-bool CRemoteListView::get_next_file(std::wstring & name, bool& dir, int64_t& size, fz::datetime& date)
+bool CRemoteListView::get_next_file(std::wstring_view & name, std::wstring & path, bool& dir, int64_t& size, fz::datetime& date)
 {
 	if (++m_comparisonIndex >= (int)m_originalIndexMapping.size()) {
 		return false;
