@@ -14,16 +14,9 @@ DEFINE_EVENT_TYPE(fzEVT_GRANTEXCLUSIVEENGINEACCESS)
 int CCommandQueue::m_requestIdCounter = 0;
 
 CCommandQueue::CCommandQueue(CFileZillaEngine *pEngine, CMainFrame* pMainFrame, CState& state)
-	: m_state(state)
-{
-	m_pEngine = pEngine;
-	m_pMainFrame = pMainFrame;
-	m_exclusiveEngineRequest = false;
-	m_exclusiveEngineLock = false;
-	m_requestId = 0;
-}
-
-CCommandQueue::~CCommandQueue()
+	: m_pEngine(pEngine)
+	, m_pMainFrame(pMainFrame)
+	, m_state(state)
 {
 }
 
@@ -36,7 +29,7 @@ bool CCommandQueue::Idle(command_origin origin) const
 	if (origin == any) {
 		return m_CommandList.empty();
 	}
-	
+
 	return std::find_if(m_CommandList.begin(), m_CommandList.end(), [origin](CommandInfo const& c) { return c.origin == origin; }) == m_CommandList.end();
 }
 
@@ -57,14 +50,17 @@ void CCommandQueue::ProcessCommand(CCommand *pCommand, CCommandQueue::command_or
 
 void CCommandQueue::ProcessNextCommand()
 {
-	if (m_inside_commandqueue)
+	if (m_inside_commandqueue) {
 		return;
+	}
 
-	if (m_exclusiveEngineLock)
+	if (m_exclusiveEngineLock) {
 		return;
+	}
 
-	if (m_pEngine->IsBusy())
+	if (m_pEngine->IsBusy()) {
 		return;
+	}
 
 	++m_inside_commandqueue;
 
@@ -91,23 +87,28 @@ void CCommandQueue::ProcessNextCommand()
 	--m_inside_commandqueue;
 
 	if (m_CommandList.empty()) {
-		if (m_exclusiveEngineRequest)
+		if (m_exclusiveEngineRequest) {
 			GrantExclusiveEngineRequest();
-		else
+		}
+		else {
 			m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
+		}
 
-		if (!m_state.SuccessfulConnect())
-			m_state.SetServer(0);
+		if (!m_state.SuccessfulConnect()) {
+			m_state.SetSite(Site());
+		}
 	}
 }
 
 bool CCommandQueue::Cancel()
 {
-	if (m_exclusiveEngineLock)
+	if (m_exclusiveEngineLock) {
 		return false;
+	}
 
-	if (m_CommandList.empty())
+	if (m_CommandList.empty()) {
 		return true;
+	}
 
 	m_CommandList.erase(++m_CommandList.begin(), m_CommandList.end());
 
@@ -118,8 +119,9 @@ bool CCommandQueue::Cancel()
 	}
 
 	int res = m_pEngine->Cancel();
-	if (res == FZ_REPLY_WOULDBLOCK)
+	if (res == FZ_REPLY_WOULDBLOCK) {
 		return false;
+	}
 	else {
 		m_CommandList.clear();
 		m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
@@ -143,12 +145,9 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 		return;
 	}
 	if (nReplyCode & FZ_REPLY_DISCONNECTED) {
-		if (commandId == Command::none && !m_CommandList.empty()) {
-			// Pending event, has no relevance during command execution
-			return;
+		if (nReplyCode & FZ_REPLY_PASSWORDFAILED) {
+			CLoginManager::Get().CachedPasswordFailed(m_state.GetSite().server);
 		}
-		if (nReplyCode & FZ_REPLY_PASSWORDFAILED)
-			CLoginManager::Get().CachedPasswordFailed(*m_state.GetServer());
 	}
 
 	if (m_CommandList.empty()) {
@@ -159,23 +158,11 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 		commandId != Command::disconnect &&
 		(nReplyCode & FZ_REPLY_CANCELED) != FZ_REPLY_CANCELED)
 	{
-		bool reconnect = false;
 		if (nReplyCode == FZ_REPLY_NOTCONNECTED) {
-			reconnect = true;
-		}
-		else if (nReplyCode & FZ_REPLY_DISCONNECTED) {
-			auto & info = m_CommandList.front();
-			if (!info.didReconnect) {
-				info.didReconnect = true;
-				reconnect = true;
-			}
-		}
-
-		if (reconnect) {
 			// Try automatic reconnect
-			const CServer* pServer = m_state.GetServer();
-			if (pServer) {
-				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(*pServer));
+			Site const& site = m_state.GetSite();
+			if (site) {
+				m_CommandList.emplace_front(normal, std::make_unique<CConnectCommand>(site.server, site.Handle(), site.credentials));
 				ProcessNextCommand();
 				return;
 			}
@@ -187,7 +174,7 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 	auto const& commandInfo = m_CommandList.front();
 
 	if (commandInfo.command->GetId() == Command::list && nReplyCode != FZ_REPLY_OK) {
-		if (nReplyCode & FZ_REPLY_LINKNOTDIR) {
+		if ((nReplyCode & FZ_REPLY_LINKNOTDIR) == FZ_REPLY_LINKNOTDIR) {
 			// Symbolic link does not point to a directory. Either points to file
 			// or is completely invalid
 			CListCommand* pListCommand = static_cast<CListCommand*>(commandInfo.command.get());
@@ -226,8 +213,9 @@ void CCommandQueue::ProcessReply(int nReplyCode, Command commandId)
 		m_state.SetSuccessfulConnect();
 		m_CommandList.pop_front();
 	}
-	else
+	else {
 		m_CommandList.pop_front();
+	}
 
 	--m_inside_commandqueue;
 
@@ -238,23 +226,21 @@ void CCommandQueue::RequestExclusiveEngine(bool requestExclusive)
 {
 	wxASSERT(!m_exclusiveEngineLock || !requestExclusive);
 
-	if (!m_exclusiveEngineRequest && requestExclusive)
-	{
+	if (!m_exclusiveEngineRequest && requestExclusive) {
 		m_requestId = ++m_requestIdCounter;
-		if (m_requestId < 0)
-		{
+		if (m_requestId < 0) {
 			m_requestIdCounter = 0;
 			m_requestId = 0;
 		}
-		if (m_CommandList.empty())
-		{
+		if (m_CommandList.empty()) {
 			m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 			GrantExclusiveEngineRequest();
 			return;
 		}
 	}
-	if (!requestExclusive)
+	if (!requestExclusive) {
 		m_exclusiveEngineLock = false;
+	}
 	m_exclusiveEngineRequest = requestExclusive;
 	m_state.NotifyHandlers(STATECHANGE_REMOTE_IDLE);
 }
@@ -272,11 +258,13 @@ void CCommandQueue::GrantExclusiveEngineRequest()
 
 CFileZillaEngine* CCommandQueue::GetEngineExclusive(int requestId)
 {
-	if (!m_exclusiveEngineLock)
+	if (!m_exclusiveEngineLock) {
 		return 0;
+	}
 
-	if (requestId != m_requestId)
+	if (requestId != m_requestId) {
 		return 0;
+	}
 
 	return m_pEngine;
 }
@@ -314,11 +302,15 @@ void CCommandQueue::ProcessDirectoryListing(CDirectoryListingNotification const&
 	}
 
 	if (listingIsRecursive) {
-		if (!listingNotification.Modified() && m_state.GetRemoteRecursiveOperation()->IsActive()) {
-			m_state.NotifyHandlers(STATECHANGE_REMOTE_DIR_OTHER, wxString(), &pListing);
+		if (listingNotification.Primary() && m_state.GetRemoteRecursiveOperation()->IsActive()) {
+			m_state.NotifyHandlers(STATECHANGE_REMOTE_DIR_OTHER, std::wstring(), &pListing);
 		}
 	}
 	else {
-		m_state.SetRemoteDir(pListing, listingNotification.Modified());
+		m_state.SetRemoteDir(pListing, listingNotification.Primary());
+	}
+
+	if (pListing && !listingNotification.Failed() && m_state.GetSite()) {
+		CContextManager::Get()->ProcessDirectoryListing(m_state.GetSite().server, pListing, listingIsRecursive ? 0 : &m_state);
 	}
 }
