@@ -11,15 +11,13 @@ CComparableListing::CComparableListing(wxWindow* pParent)
 
 	// Init backgrounds for directory comparison
 	wxColour background = m_pParent->GetBackgroundColour();
-	if (background.Red() + background.Green() + background.Blue() >= 384)
-	{
+	if (background.Red() + background.Green() + background.Blue() >= 384) {
 		// Light background
 		m_comparisonBackgrounds[0].SetBackgroundColour(wxColour(255, 128, 128));
 		m_comparisonBackgrounds[1].SetBackgroundColour(wxColour(255, 255, 128));
 		m_comparisonBackgrounds[2].SetBackgroundColour(wxColour(128, 255, 128));
 	}
-	else
-	{
+	else {
 		// Light background
 		m_comparisonBackgrounds[0].SetBackgroundColour(wxColour(192, 64, 64));
 		m_comparisonBackgrounds[1].SetBackgroundColour(wxColour(192, 192, 64));
@@ -31,31 +29,33 @@ CComparableListing::CComparableListing(wxWindow* pParent)
 
 bool CComparableListing::IsComparing() const
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return false;
+	}
 
 	return m_pComparisonManager->IsComparing();
 }
 
 void CComparableListing::ExitComparisonMode()
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return;
+	}
 
 	m_pComparisonManager->ExitComparisonMode();
 }
 
 void CComparableListing::RefreshComparison()
 {
-	if (!m_pComparisonManager)
+	if (!m_pComparisonManager) {
 		return;
+	}
 
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
-	if (!CanStartComparison(0) || !GetOther() || !GetOther()->CanStartComparison(0))
-	{
-		ExitComparisonMode();
+	if (!CanStartComparison() || !GetOther() || !GetOther()->CanStartComparison()) {
 		return;
 	}
 
@@ -64,139 +64,117 @@ void CComparableListing::RefreshComparison()
 
 bool CComparisonManager::CompareListings()
 {
-	if (!m_pLeft || !m_pRight)
+	if (!m_pLeft || !m_pRight) {
 		return false;
+	}
 
 	CFilterManager filters;
-	if (filters.HasActiveFilters() && !filters.HasSameLocalAndRemoteFilters())
-	{
-		m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
+	if (filters.HasActiveFilters() && !filters.HasSameLocalAndRemoteFilters()) {
+		m_state.NotifyHandlers(STATECHANGE_COMPARISON);
 		wxMessageBoxEx(_("Cannot compare directories, different filters for local and remote directories are enabled"), _("Directory comparison failed"), wxICON_EXCLAMATION);
 		return false;
 	}
 
-	wxString error;
-	if (!m_pLeft->CanStartComparison(&error))
-	{
-		m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
-		wxMessageBoxEx(error, _("Directory comparison failed"), wxICON_EXCLAMATION);
-		return false;
-	}
-	if (!m_pRight->CanStartComparison(&error))
-	{
-		m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
-		wxMessageBoxEx(error, _("Directory comparison failed"), wxICON_EXCLAMATION);
-		return false;
-	}
-
-	const int mode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
-	const wxTimeSpan threshold = wxTimeSpan::Minutes( COptions::Get()->GetOptionVal(OPTION_COMPARISON_THRESHOLD) );
-
+	m_isComparing = true;
 	m_pLeft->m_pComparisonManager = this;
 	m_pRight->m_pComparisonManager = this;
 
-	m_isComparing = true;
+	m_state.NotifyHandlers(STATECHANGE_COMPARISON);
 
-	m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
+	if (!m_pLeft->CanStartComparison() || !m_pRight->CanStartComparison()) {
+		return true;
+	}
+
+	fz::duration const threshold = fz::duration::from_minutes( COptions::Get()->GetOptionVal(OPTION_COMPARISON_THRESHOLD) );
 
 	m_pLeft->StartComparison();
 	m_pRight->StartComparison();
 
-	wxString localFile, remoteFile;
+	std::wstring localPath, remotePath;
+	std::wstring_view localFile, remoteFile;
 	bool localDir = false;
 	bool remoteDir = false;
-	wxLongLong localSize, remoteSize;
-	CDateTime localDate, remoteDate;
+	int64_t localSize, remoteSize;
+	fz::datetime localDate, remoteDate;
 
 	const int dirSortMode = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
 
-	const bool hide_identical = COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0;
+	bool gotLocal = m_pLeft->get_next_file(localFile, localPath, localDir, localSize, localDate);
+	bool gotRemote = m_pRight->get_next_file(remoteFile, remotePath, remoteDir, remoteSize, remoteDate);
 
-	bool gotLocal = m_pLeft->GetNextFile(localFile, localDir, localSize, localDate);
-	bool gotRemote = m_pRight->GetNextFile(remoteFile, remoteDir, remoteSize, remoteDate);
-
-	while (gotLocal && gotRemote)
-	{
-		int cmp = CompareFiles(dirSortMode, localFile, remoteFile, localDir, remoteDir);
-		if (!cmp)
-		{
-			if (!mode)
-			{
+	while (gotLocal && gotRemote) {
+		int cmp = CompareFiles(dirSortMode, localPath, localFile, remotePath, remoteFile, localDir, remoteDir);
+		if (!cmp) {
+			if (!m_comparisonMode) {
 				const CComparableListing::t_fileEntryFlags flag = (localDir || localSize == remoteSize) ? CComparableListing::normal : CComparableListing::different;
 
-				if (!hide_identical || flag != CComparableListing::normal || localFile == _T(".."))
-				{
+				if (!m_hideIdentical || flag != CComparableListing::normal || localFile == L"..") {
 					m_pLeft->CompareAddFile(flag);
 					m_pRight->CompareAddFile(flag);
 				}
 			}
-			else
-			{
-				if (!localDate.IsValid() || !remoteDate.IsValid())
-				{
-					if (!hide_identical || localDate.IsValid() || remoteDate.IsValid() || localFile == _T(".."))
-					{
+			else {
+				if (localDate.empty() || remoteDate.empty()) {
+					if (!m_hideIdentical || !localDate.empty() || !remoteDate.empty() || localFile == L"..") {
 						const CComparableListing::t_fileEntryFlags flag = CComparableListing::normal;
 						m_pLeft->CompareAddFile(flag);
 						m_pRight->CompareAddFile(flag);
 					}
 				}
-				else
-				{
+				else {
 					CComparableListing::t_fileEntryFlags localFlag, remoteFlag;
 
-					int cmp = localDate.Compare(remoteDate);
-					if( cmp < 0 )
+					int dateCmp = localDate.compare(remoteDate);
+					if (dateCmp < 0) {
 						localDate += threshold;
-					else if( cmp > 0 ) {
+					}
+					else if (dateCmp > 0 ) {
 						remoteDate += threshold;
 					}
-					int cmp2 = localDate.Compare(remoteDate);
-					if( cmp && cmp == -cmp2) {
-						cmp = 0;
+					int adjustedDateCmp = localDate.compare(remoteDate);
+					if (dateCmp && dateCmp == -adjustedDateCmp) {
+						dateCmp = 0;
 					}
 
 					localFlag = CComparableListing::normal;
 					remoteFlag = CComparableListing::normal;
-					if( cmp < 0 ) {
+					if (dateCmp < 0 ) {
 						remoteFlag = CComparableListing::newer;
 					}
-					else if( cmp > 0 ) {
+					else if (dateCmp > 0) {
 						localFlag = CComparableListing::newer;
 					}
-					if (!hide_identical || localFlag != CComparableListing::normal || remoteFlag != CComparableListing::normal || localFile == _T(".."))
-					{
+					if (!m_hideIdentical || localFlag != CComparableListing::normal || remoteFlag != CComparableListing::normal || localFile == L"..") {
 						m_pLeft->CompareAddFile(localFlag);
 						m_pRight->CompareAddFile(remoteFlag);
 					}
 				}
 			}
-			gotLocal = m_pLeft->GetNextFile(localFile, localDir, localSize, localDate);
-			gotRemote = m_pRight->GetNextFile(remoteFile, remoteDir, remoteSize, remoteDate);
+			gotLocal = m_pLeft->get_next_file(localFile, localPath, localDir, localSize, localDate);
+			gotRemote = m_pRight->get_next_file(remoteFile, remotePath, remoteDir, remoteSize, remoteDate);
 			continue;
 		}
 
 		if (cmp < 0) {
 			m_pLeft->CompareAddFile(CComparableListing::lonely);
 			m_pRight->CompareAddFile(CComparableListing::fill);
-			gotLocal = m_pLeft->GetNextFile(localFile, localDir, localSize, localDate);
+			gotLocal = m_pLeft->get_next_file(localFile, localPath, localDir, localSize, localDate);
 		}
 		else {
 			m_pLeft->CompareAddFile(CComparableListing::fill);
 			m_pRight->CompareAddFile(CComparableListing::lonely);
-			gotRemote = m_pRight->GetNextFile(remoteFile, remoteDir, remoteSize, remoteDate);
+			gotRemote = m_pRight->get_next_file(remoteFile, remotePath, remoteDir, remoteSize, remoteDate);
 		}
 	}
 	while (gotLocal) {
 		m_pLeft->CompareAddFile(CComparableListing::lonely);
 		m_pRight->CompareAddFile(CComparableListing::fill);
-		gotLocal = m_pLeft->GetNextFile(localFile, localDir, localSize, localDate);
+		gotLocal = m_pLeft->get_next_file(localFile, localPath, localDir, localSize, localDate);
 	}
-	while (gotRemote)
-	{
+	while (gotRemote) {
 		m_pLeft->CompareAddFile(CComparableListing::fill);
 		m_pRight->CompareAddFile(CComparableListing::lonely);
-		gotRemote = m_pRight->GetNextFile(remoteFile, remoteDir, remoteSize, remoteDate);
+		gotRemote = m_pRight->get_next_file(remoteFile, remotePath, remoteDir, remoteSize, remoteDate);
 	}
 
 	m_pRight->FinishComparison();
@@ -205,18 +183,19 @@ bool CComparisonManager::CompareListings()
 	return true;
 }
 
-int CComparisonManager::CompareFiles(const int dirSortMode, const wxString& local, const wxString& remote, bool localDir, bool remoteDir)
+int CComparisonManager::CompareFiles(const int dirSortMode, std::wstring_view const& local_path, std::wstring_view const& local, std::wstring_view const& remote_path, std::wstring_view const& remote, bool localDir, bool remoteDir)
 {
 	switch (dirSortMode)
 	{
 	default:
-		if (localDir)
-		{
-			if (!remoteDir)
+		if (localDir) {
+			if (!remoteDir) {
 				return -1;
+			}
 		}
-		else if (remoteDir)
+		else if (remoteDir) {
 			return 1;
+		}
 		break;
 	case 2:
 		// Inline
@@ -224,51 +203,66 @@ int CComparisonManager::CompareFiles(const int dirSortMode, const wxString& loca
 	}
 
 #ifdef __WXMSW__
-	return local.CmpNoCase(remote);
+	auto cmp = fz::stricmp(local, remote);
+	if (!cmp) {
+		return fz::stricmp(local_path, remote_path);
+	}
+	return cmp;
 #else
-	return local.Cmp(remote);
+	auto cmp = local.compare(remote);
+	if (!cmp) {
+		return local_path.compare(remote_path);
+	}
+	return cmp;
 #endif
-
-	return 0;
 }
 
-CComparisonManager::CComparisonManager(CState* pState)
-	: m_pState(pState), m_pLeft(0), m_pRight(0)
+CComparisonManager::CComparisonManager(CState& state)
+	: m_state(state)
 {
-	m_isComparing = false;
+	m_comparisonMode = COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE);
+	m_hideIdentical = COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0;
 }
 
 void CComparisonManager::SetListings(CComparableListing* pLeft, CComparableListing* pRight)
 {
 	wxASSERT((pLeft && pRight) || (!pLeft && !pRight));
 
-	if (IsComparing())
+	if (IsComparing()) {
 		ExitComparisonMode();
+	}
 
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->SetOther(0);
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->SetOther(0);
+	}
 
 	m_pLeft = pLeft;
 	m_pRight = pRight;
 
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->SetOther(m_pRight);
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->SetOther(m_pLeft);
+	}
 }
 
 void CComparisonManager::ExitComparisonMode()
 {
-	if (!IsComparing())
+	if (!IsComparing()) {
 		return;
+	}
 
 	m_isComparing = false;
-	if (m_pLeft)
+	if (m_pLeft) {
 		m_pLeft->OnExitComparisonMode();
-	if (m_pRight)
+	}
+	if (m_pRight) {
 		m_pRight->OnExitComparisonMode();
+	}
 
-	m_pState->NotifyHandlers(STATECHANGE_COMPARISON);
+	m_state.NotifyHandlers(STATECHANGE_COMPARISON);
 }
