@@ -65,7 +65,7 @@ bool CSettingsDialog::Create(CMainFrame* pMainFrame)
 
 	left->Add(new wxStaticText(this, -1, _("Select &page:")));
 
-	tree_ = new wxTreeCtrlEx(this, XRCID("ID_TREE"), wxDefaultPosition, wxDefaultSize, wxTR_HAS_BUTTONS | wxTR_LINES_AT_ROOT | wxTR_HIDE_ROOT);
+	tree_ = new wxTreeCtrlEx(this, XRCID("ID_TREE"), wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTR_HIDE_ROOT);
 	tree_->SetFocus();
 	left->Add(tree_, 1, wxGROW);
 
@@ -193,7 +193,10 @@ bool CSettingsDialog::LoadPages()
 	// Keep track of maximum page size
 	size = wxSize(0, 0);
 	for (auto const& page : m_pages) {
-		size.IncTo(page.page->GetSizer()->GetMinSize());
+		auto sizer = page.page->GetSizer();
+		if (sizer) {
+			size.IncTo(sizer->GetMinSize());
+		}
 	}
 
 	wxSize panelSize = size;
@@ -204,9 +207,12 @@ bool CSettingsDialog::LoadPages()
 
 	// Adjust pages sizes according to maximum size
 	for (auto const& page : m_pages) {
-		page.page->GetSizer()->SetMinSize(size);
-		page.page->GetSizer()->Fit(page.page);
-		page.page->GetSizer()->SetSizeHints(page.page);
+		auto sizer = page.page->GetSizer();
+		if (sizer) {
+			sizer->SetMinSize(size);
+			sizer->Fit(page.page);
+			sizer->SetSizeHints(page.page);
+		}
 		if (GetLayoutDirection() == wxLayout_RightToLeft) {
 			page.page->Move(wxPoint(1, 0));
 		}
