@@ -11,6 +11,7 @@
 #include "Options.h"
 #include "sitemanager_site.h"
 #include "themeprovider.h"
+#include "textctrlex.h"
 #include "treectrlex.h"
 #include "window_state_manager.h"
 #include "wrapengine.h"
@@ -293,11 +294,11 @@ wxPanel * CreateBookmarkPanel(wxWindow* parent, DialogLayout const& lay)
 	auto row = lay.createFlex(0, 1);
 	main->Add(row, lay.grow);
 	row->AddGrowableCol(0);
-	row->Add(new wxTextCtrl(panel, XRCID("ID_BOOKMARK_LOCALDIR")), lay.valigng);
+	row->Add(new wxTextCtrlEx(panel, XRCID("ID_BOOKMARK_LOCALDIR")), lay.valigng);
 	row->Add(new wxButton(panel, XRCID("ID_BOOKMARK_BROWSE"), _("&Browse...")), lay.valign);
 	main->AddSpacer(0);
 	main->Add(new wxStaticText(panel, -1, _("&Remote directory:")));
-	main->Add(new wxTextCtrl(panel, XRCID("ID_BOOKMARK_REMOTEDIR")), lay.grow);
+	main->Add(new wxTextCtrlEx(panel, XRCID("ID_BOOKMARK_REMOTEDIR")), lay.grow);
 	main->AddSpacer(0);
 	main->Add(new wxCheckBox(panel, XRCID("ID_BOOKMARK_SYNC"), _("Use &synchronized browsing")));
 	main->Add(new wxCheckBox(panel, XRCID("ID_BOOKMARK_COMPARISON"), _("Directory comparison")));
@@ -340,7 +341,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 
 	left->Add(new wxStaticText(this, wxID_ANY, _("&Select entry:")));
 
-	tree_ = new wxTreeCtrlEx(this, XRCID("ID_SITETREE"), wxDefaultPosition, wxDefaultSize, wxBORDER_SUNKEN | wxTR_EDIT_LABELS | wxTR_HAS_BUTTONS | wxTR_MULTIPLE | wxTR_LINES_AT_ROOT);
+	tree_ = new wxTreeCtrlEx(this, XRCID("ID_SITETREE"), wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxBORDER_SUNKEN | wxTR_EDIT_LABELS | wxTR_MULTIPLE);
 	tree_->SetFocus();
 	left->Add(tree_, lay.grow)->SetProportion(1);
 
@@ -446,10 +447,7 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 	}
 #endif
 
-	wxAcceleratorEntry accel(0, WXK_F3, XRCID("ID_SEARCH"));
-	wxAcceleratorTable accelTable(1, &accel);
-	SetAcceleratorTable(accelTable);
-
+	acceleratorTable_.emplace_back(0, WXK_F3, XRCID("ID_SEARCH"));
 
 	m_connected_sites = connected_sites;
 	MarkConnectedSites();
@@ -631,7 +629,7 @@ public:
 			data->credentials.encrypted_ = fz::public_key();
 		}
 
-		std::wstring const name = data->server.GetName();
+		std::wstring const name = data->GetName();
 
 		CSiteManagerItemData* pData = new CSiteManagerItemData(std::move(data));
 		wxTreeItemId newItem = m_tree_->AppendItem(m_item, name, 2, 2, pData);
@@ -870,10 +868,8 @@ bool CSiteManagerDialog::SaveChild(pugi::xml_node element, wxTreeItemId child)
 		CSiteManager::Save(node, *data->m_site);
 
 		if (data->connected_item != -1) {
-			if ((*m_connected_sites)[data->connected_item].site.server == data->m_site->server) {
-				(*m_connected_sites)[data->connected_item].new_path = GetSitePath(child);
-				(*m_connected_sites)[data->connected_item].site = *data->m_site;
-			}
+			(*m_connected_sites)[data->connected_item].site = *data->m_site;
+			(*m_connected_sites)[data->connected_item].site.SetSitePath(GetSitePath(child));
 		}
 	}
 
@@ -924,8 +920,8 @@ bool CSiteManagerDialog::Verify()
 	}
 
 	if (data->m_site) {
-		bool const predefined = IsPredefinedItem(item);
-		if (!m_pNotebook_Site->Verify(predefined)) {
+		Site newSite = *data->m_site;
+		if (!m_pNotebook_Site->UpdateSite(newSite, false)) {
 			return false;
 		}
 	}
@@ -1142,12 +1138,18 @@ void CSiteManagerDialog::OnSelChanging(wxTreeEvent& event)
 	UpdateItem();
 }
 
-void CSiteManagerDialog::OnSelChanged(wxTreeEvent&)
+void CSiteManagerDialog::OnSelChanged(wxTreeEvent& evt)
 {
 	if (m_is_deleting) {
 		return;
 	}
 
+	if (tree_->InPrefixSearch()) {
+		m_is_deleting = true;
+		tree_->SafeSelectItem(evt.GetItem());
+		m_is_deleting = false;
+	}
+
 	SetCtrlState();
 }
 
@@ -1193,8 +1195,7 @@ bool CSiteManagerDialog::UpdateItem()
 	}
 
 	if (data->m_site) {
-		UpdateServer(*data->m_site, tree_->GetItemText(item));
-		return true;
+		return UpdateServer(*data->m_site, tree_->GetItemText(item));
 	}
 	else {
 		wxASSERT(data->m_bookmark);
@@ -1220,11 +1221,16 @@ bool CSiteManagerDialog::UpdateBookmark(Bookmark &bookmark, Site const& site)
 	return true;
 }
 
-void CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
+bool CSiteManagerDialog::UpdateServer(Site & site, const wxString &name)
 {
-	m_pNotebook_Site->UpdateSite(site);
+	Site newSite = site;
+	newSite.SetName(name.ToStdWstring());
+	if (!m_pNotebook_Site->UpdateSite(newSite, true)) {
+		return false;
+	}
 
-	site.server.SetName(name.ToStdWstring());
+	site = newSite;
+	return true;
 }
 
 bool CSiteManagerDialog::GetServer(Site& data, Bookmark& bookmark)
@@ -1838,6 +1844,10 @@ void CSiteManagerDialog::AddNewSite(wxTreeItemId parent, Site const& site, bool
 	CSiteManagerItemData* pData = new CSiteManagerItemData;
 	pData->m_site = std::make_unique<Site>();
 	*pData->m_site = site;
+
+	// Erase updated server info
+	pData->m_site->server = site.GetOriginalServer();
+	pData->m_site->originalServer.reset();
 	if (connected) {
 		pData->connected_item = 0;
 	}
@@ -1879,7 +1889,7 @@ void CSiteManagerDialog::RememberLastSelected()
 	COptions::Get()->SetOption(OPTION_SITEMANAGER_LASTSELECTED, path);
 }
 
-void CSiteManagerDialog::OnContextMenu(wxTreeEvent& event)
+void CSiteManagerDialog::OnContextMenu(wxTreeEvent&)
 {
 	if (!Verify()) {
 		return;
