@@ -15,6 +15,8 @@
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/hash.hpp>
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/signature.hpp>
+#include <libfilezilla/translate.hpp>
 
 #include <string>
 
@@ -125,9 +127,18 @@ CUpdater* CUpdater::GetInstance()
 void CUpdater::AutoRunIfNeeded()
 {
 #if FZ_AUTOUPDATECHECK
-	if (state_ == UpdaterState::failed || state_ == UpdaterState::idle) {
-		if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0 && LongTimeSinceLastCheck()) {
-			Run(false);
+	if (state_ == UpdaterState::failed || state_ == UpdaterState::idle || state_ == UpdaterState::newversion_stale) {
+		if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->GetOptionVal(OPTION_UPDATECHECK) != 0) {
+			if (LongTimeSinceLastCheck()) {
+				Run(false);
+			}
+		}
+		else {
+			auto const age = fz::datetime::now() - CBuildInfo::GetBuildDate();
+			if (age >= fz::duration::from_days(31*6)) {
+				version_information_ = version_information();
+				SetState(UpdaterState::newversion_stale);
+			}
 		}
 	}
 #endif
@@ -136,7 +147,8 @@ void CUpdater::AutoRunIfNeeded()
 void CUpdater::RunIfNeeded()
 {
 	build const b = AvailableBuild();
-	if (state_ == UpdaterState::idle || state_ == UpdaterState::failed ||
+	if (state_ == UpdaterState::idle || state_ == UpdaterState::failed || state_ == UpdaterState::newversion_stale ||
+		state_ == UpdaterState::eol ||
 		LongTimeSinceLastCheck() || (state_ == UpdaterState::newversion && !b.url_.empty()) ||
 		(state_ == UpdaterState::newversion_ready && !VerifyChecksum(DownloadedFile(), b.size_, b.hash_)))
 	{
@@ -251,7 +263,8 @@ fz::uri CUpdater::GetUrl()
 bool CUpdater::Run(bool manual)
 {
 	if (state_ != UpdaterState::idle && state_ != UpdaterState::failed &&
-		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready)
+		state_ != UpdaterState::newversion && state_ != UpdaterState::newversion_ready
+		&& state_ != UpdaterState::newversion_stale && state_ != UpdaterState::eol)
 	{
 		return false;
 	}
@@ -398,19 +411,19 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 			else if (pData->GetRequestID() == reqId_certificate) {
 				auto & certNotification = static_cast<CCertificateNotification &>(*pData.get());
 				if (m_use_internal_rootcert) {
-					auto certs = certNotification.GetCertificates();
+					auto certs = certNotification.info_.get_certificates();
 					if (certs.size() > 1) {
 						auto const& ca = certs.back();
-						std::vector<uint8_t> ca_data = ca.GetRawData();
+						std::vector<uint8_t> ca_data = ca.get_raw_data();
 
-						std::string updater_root = fz::base64_decode(s_update_cert);
-						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size())) {
-							certNotification.m_trusted = true;
+						auto const updater_root = fz::base64_decode(s_update_cert);
+						if (ca_data == updater_root) {
+							certNotification.trusted_ = true;
 						}
 					}
 				}
 				else {
-					certNotification.m_trusted = true;
+					certNotification.trusted_ = true;
 				}
 			}
 			engine_->SetAsyncRequestReply(std::move(pData));
@@ -439,7 +452,10 @@ UpdaterState CUpdater::ProcessFinishedData(bool can_download)
 
 	ParseData();
 
-	if (version_information_.available_.version_.empty()) {
+	if (version_information_.eol_) {
+		s = UpdaterState::eol;
+	}
+	else if (version_information_.available_.version_.empty()) {
 		s = UpdaterState::idle;
 	}
 	else if (!version_information_.available_.url_.empty()) {
@@ -664,7 +680,36 @@ void CUpdater::ParseData()
 			}
 			continue;
 		}
+		else if (type == L"eol") {
+#if defined(__WXMSW__) || defined(__WXMAC__)
+			std::string host = fz::to_utf8(CBuildInfo::GetHostname());
+			if (host.empty()) {
+				host = "unknown";
+			}
+			fz::to_utf8(CBuildInfo::GetVersion());
 
+			std::string data = host + '|' + fz::to_utf8(CBuildInfo::GetVersion()) + '|' + fz::sprintf("%d.%d", wxPlatformInfo::Get().GetOSMajorVersion(), wxPlatformInfo::Get().GetOSMinorVersion());
+
+			bool valid_signature{};
+			for (size_t i = 1; i < tokens.size(); ++i) {
+				auto const& token = tokens[i];
+				if (token.substr(0, 4) == "sig:") {
+					auto const& sig = token.substr(4);
+					auto raw_sig = fz::base64_decode_s(fz::to_utf8(sig));
+
+					if (!raw_sig.empty()) {
+						auto const pub = fz::public_verification_key::from_base64("xrjuitldZT7pvIhK9q1GVNfptrepB/ctt5aK1QO5RaI");
+						valid_signature = fz::verify(data, raw_sig, pub);
+					}
+				}
+			}
+			if (!valid_signature) {
+				log_ += fz::sprintf(L"Ignoring eol statement not matching our version and platform.\n");
+				continue;
+			}
+			version_information_.eol_ = true;
+#endif
+		}
 
 		std::wstring const& versionOrDate = tokens[1];
 
@@ -726,11 +771,34 @@ void CUpdater::ParseData()
 			auto const size = fz::to_integral<uint64_t>(sizestr);
 			if (!size) {
 				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-					log_ += fz::sprintf(L"Could not parse size: %s", sizestr) + L"\n";
+					log_ += fz::sprintf(L"Could not parse size: %s\n", sizestr);
 				}
 				continue;
 			}
 
+			bool valid_signature{};
+			for (size_t i = 6; i < tokens.size(); ++i) {
+				auto const& token = tokens[i];
+				if (token.substr(0, 4) == "sig:") {
+					auto const& sig = token.substr(4);
+					auto raw_sig = fz::base64_decode(fz::to_utf8(sig));
+					auto raw_hash = fz::hex_decode(hash);
+
+					// Append the version to the file hash to protect against replays
+					raw_hash.push_back(0);
+					raw_hash.insert(raw_hash.cend(), versionOrDate.cbegin(), versionOrDate.cend());
+
+					if (!raw_sig.empty() || !raw_hash.empty()) {
+						auto const pub = fz::public_verification_key::from_base64("xrjuitldZT7pvIhK9q1GVNfptrepB/ctt5aK1QO5RaI");
+						valid_signature = fz::verify(raw_hash, raw_sig, pub);
+					}
+				}
+			}
+			if (!valid_signature) {
+				log_ += fz::sprintf(L"Ignoring line with inalid or missing signature for hash %s\n", hash);
+				continue;
+			}
+
 			b.url_ = url;
 			b.size_ = size;
 			b.hash_ = fz::str_tolower_ascii(hash);
