@@ -17,21 +17,21 @@ CertStore::CertStore()
 {
 }
 
-bool CertStore::IsTrusted(CCertificateNotification const& notification)
+bool CertStore::IsTrusted(fz::tls_session_info const& info)
 {
-	if (notification.GetAlgorithmWarnings() != 0) {
+	if (info.get_algorithm_warnings() != 0) {
 		// These certs are never trusted.
 		return false;
 	}
 
 	LoadTrustedCerts();
 
-	CCertificate cert = notification.GetCertificates()[0];
+	fz::x509_certificate cert = info.get_certificates()[0];
 
-	return IsTrusted(notification.GetHost(), notification.GetPort(), cert.GetRawData(), false, !notification.MismatchedHostname());
+	return IsTrusted(info.get_host(), info.get_port(), cert.get_raw_data(), false, !info.mismatched_hostname());
 }
 
-bool CertStore::IsInsecure(std::wstring const& host, unsigned int port, bool permanentOnly)
+bool CertStore::IsInsecure(std::string const& host, unsigned int port, bool permanentOnly)
 {
 	auto const t = std::make_tuple(host, port);
 	if (!permanentOnly && sessionInsecureHosts_.find(t) != sessionInsecureHosts_.cend()) {
@@ -47,7 +47,7 @@ bool CertStore::IsInsecure(std::wstring const& host, unsigned int port, bool per
 	return false;
 }
 
-bool CertStore::HasCertificate(std::wstring const& host, unsigned int port)
+bool CertStore::HasCertificate(std::string const& host, unsigned int port)
 {
 	for (auto const& cert : sessionTrustedCerts_) {
 		if (cert.host == host && cert.port == port) {
@@ -66,14 +66,14 @@ bool CertStore::HasCertificate(std::wstring const& host, unsigned int port)
 	return false;
 }
 
-bool CertStore::DoIsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<CertStore::t_certData> const& trustedCerts, bool allowSans)
+bool CertStore::DoIsTrusted(std::string const& host, unsigned int port, std::vector<uint8_t> const& data, std::list<CertStore::t_certData> const& trustedCerts, bool allowSans)
 {
 	if (!data.size()) {
 		return false;
 	}
 
 	bool const dnsname = fz::get_address_type(host) == fz::address_type::unknown;
-	
+
 	for (auto const& cert : trustedCerts) {
 		if (port != cert.port) {
 			continue;
@@ -95,7 +95,7 @@ bool CertStore::DoIsTrusted(std::wstring const& host, unsigned int port, std::ve
 	return false;
 }
 
-bool CertStore::IsTrusted(std::wstring const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly, bool allowSans)
+bool CertStore::IsTrusted(std::string const& host, unsigned int port, std::vector<uint8_t> const& data, bool permanentOnly, bool allowSans)
 {
 	bool trusted = DoIsTrusted(host, port, data, trustedCerts_, allowSans);
 	if (!trusted && !permanentOnly) {
@@ -135,7 +135,7 @@ void CertStore::LoadTrustedCerts()
 				return false;
 			}
 
-			data.host = GetTextElement(cert, "Host");
+			data.host = cert.child_value("Host");
 			data.port = GetTextElementInt(cert, "Port");
 			if (data.host.empty() || data.port < 1 || data.port > 65535) {
 				return false;
@@ -180,7 +180,7 @@ void CertStore::LoadTrustedCerts()
 
 		auto const processEntry = [&](pugi::xml_node const& node)
 		{
-			std::wstring host = GetTextElement(node);
+			std::string host = node.child_value();
 			unsigned int port = node.attribute("Port").as_uint();
 			if (host.empty() || port < 1 || port > 65535) {
 				return false;
@@ -215,7 +215,7 @@ void CertStore::LoadTrustedCerts()
 	}
 }
 
-void CertStore::SetInsecure(std::wstring const& host, unsigned int port, bool permanent)
+void CertStore::SetInsecure(std::string const& host, unsigned int port, bool permanent)
 {
 	// A host can't be both trusted and insecure
 	sessionTrustedCerts_.erase(
@@ -243,7 +243,7 @@ void CertStore::SetInsecure(std::wstring const& host, unsigned int port, bool pe
 			// Purge certificates for this host
 			auto const processEntry = [&host, &port](pugi::xml_node const& cert)
 			{
-				return host != GetTextElement(cert, "Host") || port != GetTextElementInt(cert, "Port");
+				return host != cert.child_value("Host") || port != GetTextElementInt(cert, "Port");
 			};
 
 			auto cert = certs.child("Certificate");
@@ -278,14 +278,14 @@ void CertStore::SetInsecure(std::wstring const& host, unsigned int port, bool pe
 	insecureHosts_.emplace(std::make_tuple(host, port));
 }
 
-void CertStore::SetTrusted(CCertificateNotification const& notification, bool permanent, bool trustAllHostnames)
+void CertStore::SetTrusted(fz::tls_session_info const& info, bool permanent, bool trustAllHostnames)
 {
-	const CCertificate certificate = notification.GetCertificates()[0];
+	const fz::x509_certificate certificate = info.get_certificates()[0];
 
 	t_certData cert;
-	cert.host = notification.GetHost();
-	cert.port = notification.GetPort();
-	cert.data = certificate.GetRawData();
+	cert.host = info.get_host();
+	cert.port = info.get_port();
+	cert.data = certificate.get_raw_data();
 
 	if (trustAllHostnames) {
 		cert.trustSans = true;
@@ -318,8 +318,8 @@ void CertStore::SetTrusted(CCertificateNotification const& notification, bool pe
 
 			auto xCert = certs.append_child("Certificate");
 			AddTextElementUtf8(xCert, "Data", fz::hex_encode<std::string>(cert.data));
-			AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.GetActivationTime().get_time_t()));
-			AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.GetExpirationTime().get_time_t()));
+			AddTextElement(xCert, "ActivationTime", static_cast<int64_t>(certificate.get_activation_time().get_time_t()));
+			AddTextElement(xCert, "ExpirationTime", static_cast<int64_t>(certificate.get_expiration_time().get_time_t()));
 			AddTextElement(xCert, "Host", cert.host);
 			AddTextElement(xCert, "Port", cert.port);
 			AddTextElement(xCert, "TrustSANs", cert.trustSans ? L"1" : L"0");
@@ -352,75 +352,67 @@ void CertStore::SetTrusted(CCertificateNotification const& notification, bool pe
 }
 
 
-
-
-CVerifyCertDialog::CVerifyCertDialog(CertStore & certStore)
-	: certStore_(certStore)
-{
-}
-
-
-bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
+bool CVerifyCertDialog::DisplayCert(fz::x509_certificate const& cert)
 {
 	bool warning = false;
-	if (!cert.GetActivationTime().empty()) {
-		if (cert.GetActivationTime() > fz::datetime::now()) {
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), CTimeFormat::Format(cert.GetActivationTime())));
-			xrc_call(*pDlg, "ID_ACTIVATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
+	if (!cert.get_activation_time().empty()) {
+		if (cert.get_activation_time() > fz::datetime::now()) {
+			SetChildLabel(XRCID("ID_ACTIVATION_TIME"), wxString::Format(_("%s - Not yet valid!"), CTimeFormat::Format(cert.get_activation_time())));
+			xrc_call(*this, "ID_ACTIVATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
 			warning = true;
 		}
 		else {
-			pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), CTimeFormat::Format(cert.GetActivationTime()));
-			xrc_call(*pDlg, "ID_ACTIVATION_TIME", &wxWindow::SetForegroundColour, wxColour());
+			SetChildLabel(XRCID("ID_ACTIVATION_TIME"), CTimeFormat::Format(cert.get_activation_time()));
+			xrc_call(*this, "ID_ACTIVATION_TIME", &wxWindow::SetForegroundColour, wxColour());
 		}
 	}
 	else {
 		warning = true;
-		pDlg->SetChildLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
+		SetChildLabel(XRCID("ID_ACTIVATION_TIME"), _("Invalid date"));
 	}
 
-	if (!cert.GetExpirationTime().empty()) {
-		if (cert.GetExpirationTime() < fz::datetime::now()) {
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), CTimeFormat::Format(cert.GetExpirationTime())));
-			xrc_call(*pDlg, "ID_EXPIRATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
+	if (!cert.get_expiration_time().empty()) {
+		if (cert.get_expiration_time() < fz::datetime::now()) {
+			SetChildLabel(XRCID("ID_EXPIRATION_TIME"), wxString::Format(_("%s - Certificate expired!"), CTimeFormat::Format(cert.get_expiration_time())));
+			xrc_call(*this, "ID_EXPIRATION_TIME", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
 			warning = true;
 		}
 		else {
-			pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), CTimeFormat::Format(cert.GetExpirationTime()));
-			xrc_call(*pDlg, "ID_EXPIRATION_TIME", &wxWindow::SetForegroundColour, wxColour());
+			SetChildLabel(XRCID("ID_EXPIRATION_TIME"), CTimeFormat::Format(cert.get_expiration_time()));
+			xrc_call(*this, "ID_EXPIRATION_TIME", &wxWindow::SetForegroundColour, wxColour());
 		}
 	}
 	else {
 		warning = true;
-		pDlg->SetChildLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
+		SetChildLabel(XRCID("ID_EXPIRATION_TIME"), _("Invalid date"));
 	}
 
-	if (!cert.GetSerial().empty()) {
-		pDlg->SetChildLabel(XRCID("ID_SERIAL"), cert.GetSerial());
+	if (!cert.get_serial().empty()) {
+		SetChildLabel(XRCID("ID_SERIAL"), fz::to_wstring_from_utf8(cert.get_serial()));
 	}
 	else {
-		pDlg->SetChildLabel(XRCID("ID_SERIAL"), _("None"));
+		SetChildLabel(XRCID("ID_SERIAL"), _("None"));
 	}
 
-	pDlg->SetChildLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), cert.GetPkAlgoName(), cert.GetPkAlgoBits()));
-	pDlg->SetChildLabel(XRCID("ID_SIGNALGO"), cert.GetSignatureAlgorithm());
+	SetChildLabel(XRCID("ID_PKALGO"), wxString::Format(_("%s with %d bits"), fz::to_wstring_from_utf8(cert.get_pubkey_algorithm()), cert.get_pubkey_bits()));
+	SetChildLabel(XRCID("ID_SIGNALGO"), fz::to_wstring_from_utf8(cert.get_signature_algorithm()));
 
-	wxString const& sha256 = cert.GetFingerPrintSHA256();
-	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA256"), sha256.Left(sha256.size() / 2 + 1) + L"\n" + sha256.Mid(sha256.size() / 2 + 1));
-	pDlg->SetChildLabel(XRCID("ID_FINGERPRINT_SHA1"), cert.GetFingerPrintSHA1());
+	wxString const sha256 = fz::to_wstring_from_utf8(cert.get_fingerprint_sha256());
+	SetChildLabel(XRCID("ID_FINGERPRINT_SHA256"), sha256.Left(sha256.size() / 2 + 1) + L"\n" + sha256.Mid(sha256.size() / 2 + 1));
+	SetChildLabel(XRCID("ID_FINGERPRINT_SHA1"), fz::to_wstring_from_utf8(cert.get_fingerprint_sha1()));
 
-	ParseDN(XRCCTRL(*pDlg, "ID_ISSUER_BOX", wxStaticBox), cert.GetIssuer(), m_pIssuerSizer);
+	ParseDN(XRCCTRL(*this, "ID_ISSUER_BOX", wxStaticBox), fz::to_wstring_from_utf8(cert.get_issuer()), m_pIssuerSizer);
 
-	auto subjectPanel = XRCCTRL(*pDlg, "ID_SUBJECT_PANEL", wxScrolledWindow);
+	auto subjectPanel = XRCCTRL(*this, "ID_SUBJECT_PANEL", wxScrolledWindow);
 	subjectPanel->Freeze();
 
-	ParseDN(subjectPanel, cert.GetSubject(), m_pSubjectSizer);
+	ParseDN(subjectPanel, fz::to_wstring_from_utf8(cert.get_subject()), m_pSubjectSizer);
 
-	auto const& altNames = cert.GetAltSubjectNames();
+	auto const& altNames = cert.get_alt_subject_names();
 	if (!altNames.empty()) {
 		wxString str;
 		for (auto const& altName : altNames) {
-			str += LabelEscape(altName.name) + L"\n";
+			str += LabelEscape(fz::to_wstring_from_utf8(altName.name)) + L"\n";
 		}
 		str.RemoveLast();
 		m_pSubjectSizer->Add(new wxStaticText(subjectPanel, wxID_ANY, wxPLURAL("Alternative name:", "Alternative names:", altNames.size())));
@@ -429,7 +421,7 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 	m_pSubjectSizer->Fit(subjectPanel);
 
 	wxSize min = m_pSubjectSizer->CalcMin();
-	int const maxHeight = (line_height_ + m_pDlg->ConvertDialogToPixels(wxPoint(0, 1)).y) * 15;
+	int const maxHeight = (line_height_ + ConvertDialogToPixels(wxPoint(0, 1)).y) * 15;
 	if (min.y >= maxHeight) {
 		min.y = maxHeight;
 		min.x += wxSystemSettings::GetMetric(wxSYS_VSCROLL_X);
@@ -446,56 +438,86 @@ bool CVerifyCertDialog::DisplayCert(wxDialogEx* pDlg, const CCertificate& cert)
 
 #include <wx/scrolwin.h>
 
-bool CVerifyCertDialog::DisplayAlgorithm(int controlId, wxString name, bool insecure)
+bool CVerifyCertDialog::DisplayAlgorithm(int controlId, std::string const& name, bool insecure)
 {
+	wxString wname = fz::to_wstring_from_utf8(name);
 	if (insecure) {
-		name += L" - ";
-		name += _("Insecure algorithm!");
+		wname += L" - ";
+		wname += _("Insecure algorithm!");
 
-		auto wnd = m_pDlg->FindWindow(controlId);
+		auto wnd = FindWindow(controlId);
 		if (wnd) {
 			wnd->SetForegroundColour(wxColour(255, 0, 0));
 		}
 	}
 
-	m_pDlg->SetChildLabel(controlId, name);
+	SetChildLabel(controlId, wname);
 
 	return insecure;
 }
 
-void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notification, bool displayOnly)
+void CVerifyCertDialog::ShowVerificationDialog(CertStore & certStore, CCertificateNotification& notification)
 {
-	m_pDlg = new wxDialogEx;
-	if (!m_pDlg->Load(0, L"ID_VERIFYCERT")) {
-		wxBell();
-		delete m_pDlg;
-		m_pDlg = 0;
+	CVerifyCertDialog dlg;
+	if (!dlg.CreateVerificationDialog(notification, false)) {
 		return;
 	}
 
-	if (displayOnly) {
-		xrc_call(*m_pDlg, "ID_DESC", &wxWindow::Hide);
-		xrc_call(*m_pDlg, "ID_ALWAYS_DESC", &wxWindow::Hide);
-		xrc_call(*m_pDlg, "ID_ALWAYS", &wxWindow::Hide);
-		xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxWindow::Hide);
-		xrc_call(*m_pDlg, "wxID_CANCEL", &wxWindow::Hide);
-		m_pDlg->SetTitle(L"Certificate details");
+	int res = dlg.ShowModal();
+	if (res == wxID_OK) {
+		notification.trusted_ = true;
+
+		if (!notification.info_.get_algorithm_warnings()) {
+			bool trustSANs = dlg.sanTrustAllowed_ && xrc_call(dlg, "ID_TRUST_SANS", &wxCheckBox::GetValue);
+			bool permanent = !dlg.warning_ && xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::GetValue);
+			certStore.SetTrusted(notification.info_, permanent, trustSANs);
+		}
 	}
 	else {
-		m_pDlg->WrapText(m_pDlg, XRCID("ID_DESC"), 420);
+		notification.trusted_ = false;
+	}
+}
+
+void CVerifyCertDialog::DisplayCertificate(CCertificateNotification const& notification)
+{
+	CVerifyCertDialog dlg;
+	if (dlg.CreateVerificationDialog(notification, true)) {
+		dlg.ShowModal();
+	}
+}
+
+bool CVerifyCertDialog::CreateVerificationDialog(CCertificateNotification const& notification, bool displayOnly)
+{
+	fz::tls_session_info const& info = notification.info_;
+
+	if (!Load(0, L"ID_VERIFYCERT", L"certificate.xrc")) {
+		wxBell();
+		return false;
+	}
+
+	if (displayOnly) {
+		xrc_call(*this, "ID_DESC", &wxWindow::Hide);
+		xrc_call(*this, "ID_ALWAYS_DESC", &wxWindow::Hide);
+		xrc_call(*this, "ID_ALWAYS", &wxWindow::Hide);
+		xrc_call(*this, "ID_TRUST_SANS", &wxWindow::Hide);
+		xrc_call(*this, "wxID_CANCEL", &wxWindow::Hide);
+		SetTitle(L"Certificate details");
+	}
+	else {
+		WrapText(this, XRCID("ID_DESC"), 420);
 
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2) {
-			XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Hide();
+			XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->Hide();
 		}
 	}
 
-	m_certificates = notification.GetCertificates();
+	m_certificates = info.get_certificates();
 	if (m_certificates.size() == 1) {
-		XRCCTRL(*m_pDlg, "ID_CHAIN_DESC", wxStaticText)->Hide();
-		XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice)->Hide();
+		XRCCTRL(*this, "ID_CHAIN_DESC", wxStaticText)->Hide();
+		XRCCTRL(*this, "ID_CHAIN", wxChoice)->Hide();
 	}
 	else {
-		wxChoice* pChoice = XRCCTRL(*m_pDlg, "ID_CHAIN", wxChoice);
+		wxChoice* pChoice = XRCCTRL(*this, "ID_CHAIN", wxChoice);
 		for (unsigned int i = 0; i < m_certificates.size(); ++i) {
 			pChoice->Append(wxString::Format(L"%d", i));
 		}
@@ -504,78 +526,60 @@ void CVerifyCertDialog::ShowVerificationDialog(CCertificateNotification& notific
 		pChoice->Connect(wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(CVerifyCertDialog::OnCertificateChoice), 0, this);
 	}
 
-	if (notification.MismatchedHostname()) {
-		xrc_call(*m_pDlg, "ID_HOST", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
-		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(_("%s:%d - Hostname does not match certificate"), LabelEscape(notification.GetHost()), notification.GetPort()));
+	if (info.mismatched_hostname()) {
+		xrc_call(*this, "ID_HOST", &wxWindow::SetForegroundColour, wxColour(255, 0, 0));
+		SetChildLabel(XRCID("ID_HOST"), wxString::Format(_("%s:%d - Hostname does not match certificate"), LabelEscape(fz::to_wstring_from_utf8(info.get_host())), info.get_port()));
 	}
 	else {
-		m_pDlg->SetChildLabel(XRCID("ID_HOST"), wxString::Format(L"%s:%d", LabelEscape(notification.GetHost()), notification.GetPort()));
+		SetChildLabel(XRCID("ID_HOST"), wxString::Format(L"%s:%d", LabelEscape(fz::to_wstring_from_utf8(info.get_host())), info.get_port()));
 	}
 
-	line_height_ = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetSize().y;
+	line_height_ = XRCCTRL(*this, "ID_SUBJECT_DUMMY", wxStaticText)->GetSize().y;
 
-	m_pSubjectSizer = XRCCTRL(*m_pDlg, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
+	m_pSubjectSizer = XRCCTRL(*this, "ID_SUBJECT_DUMMY", wxStaticText)->GetContainingSizer();
 	m_pSubjectSizer->Clear(true);
 
-	m_pIssuerSizer = XRCCTRL(*m_pDlg, "ID_ISSUER_DUMMY", wxStaticText)->GetContainingSizer();
+	m_pIssuerSizer = XRCCTRL(*this, "ID_ISSUER_DUMMY", wxStaticText)->GetContainingSizer();
 	m_pIssuerSizer->Clear(true);
 
 	wxSize minSize(0, 0);
 	for (unsigned int i = 0; i < m_certificates.size(); ++i) {
-		DisplayCert(m_pDlg, m_certificates[i]);
-		m_pDlg->Layout();
-		m_pDlg->GetSizer()->Fit(m_pDlg);
-		minSize.IncTo(m_pDlg->GetSizer()->GetMinSize());
+		DisplayCert(m_certificates[i]);
+		Layout();
+		GetSizer()->Fit(this);
+		minSize.IncTo(GetSizer()->GetMinSize());
 	}
-	m_pDlg->GetSizer()->SetMinSize(minSize);
+	GetSizer()->SetMinSize(minSize);
 
-	bool warning = DisplayCert(m_pDlg, m_certificates[0]);
+	warning_ = DisplayCert(m_certificates[0]);
 
-	DisplayAlgorithm(XRCID("ID_PROTOCOL"), notification.GetProtocol(), (notification.GetAlgorithmWarnings() & CCertificateNotification::tlsver) != 0);
-	DisplayAlgorithm(XRCID("ID_KEYEXCHANGE"), notification.GetKeyExchange(), (notification.GetAlgorithmWarnings() & CCertificateNotification::kex) != 0);
-	DisplayAlgorithm(XRCID("ID_CIPHER"), notification.GetSessionCipher(), (notification.GetAlgorithmWarnings() & CCertificateNotification::cipher) != 0);
-	DisplayAlgorithm(XRCID("ID_MAC"), notification.GetSessionMac(), (notification.GetAlgorithmWarnings() & CCertificateNotification::mac) != 0);
+	DisplayAlgorithm(XRCID("ID_PROTOCOL"), info.get_protocol(), (info.get_algorithm_warnings() & fz::tls_session_info::tlsver) != 0);
+	DisplayAlgorithm(XRCID("ID_KEYEXCHANGE"), info.get_key_exchange(), (info.get_algorithm_warnings() & fz::tls_session_info::kex) != 0);
+	DisplayAlgorithm(XRCID("ID_CIPHER"), info.get_session_cipher(), (info.get_algorithm_warnings() & fz::tls_session_info::cipher) != 0);
+	DisplayAlgorithm(XRCID("ID_MAC"), info.get_session_mac(), (info.get_algorithm_warnings() & fz::tls_session_info::mac) != 0);
 
-	if (notification.GetAlgorithmWarnings() != 0) {
-		warning = true;
+	if (info.get_algorithm_warnings() != 0) {
+		warning_ = true;
 	}
 
-	if (warning) {
-		XRCCTRL(*m_pDlg, "ID_IMAGE", wxStaticBitmap)->SetBitmap(wxArtProvider::GetBitmap(wxART_WARNING));
-		XRCCTRL(*m_pDlg, "ID_ALWAYS", wxCheckBox)->Enable(false);
+	if (warning_) {
+		XRCCTRL(*this, "ID_IMAGE", wxStaticBitmap)->SetBitmap(wxArtProvider::GetBitmap(wxART_WARNING));
+		XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->Enable(false);
 	}
 
-	bool const dnsname = fz::get_address_type(notification.GetHost()) == fz::address_type::unknown;
-	bool const sanTrustAllowed = !warning && dnsname && !notification.MismatchedHostname();
-	XRCCTRL(*m_pDlg, "ID_TRUST_SANS", wxCheckBox)->Enable(sanTrustAllowed);
+	bool const dnsname = fz::get_address_type(info.get_host()) == fz::address_type::unknown;
+	sanTrustAllowed_ = !warning_ && dnsname && !info.mismatched_hostname();
+	XRCCTRL(*this, "ID_TRUST_SANS", wxCheckBox)->Enable(sanTrustAllowed_);
 
-	if (sanTrustAllowed && notification.SystemTrust()) {
-		xrc_call(*m_pDlg, "ID_ALWAYS", &wxCheckBox::SetValue, true);
-		xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxCheckBox::SetValue, true);
+	if (sanTrustAllowed_ && info.system_trust()) {
+		xrc_call(*this, "ID_ALWAYS", &wxCheckBox::SetValue, true);
+		xrc_call(*this, "ID_TRUST_SANS", &wxCheckBox::SetValue, true);
 	}
 
-	m_pDlg->GetSizer()->Fit(m_pDlg);
-	m_pDlg->GetSizer()->SetSizeHints(m_pDlg);
+	GetSizer()->Fit(this);
+	GetSizer()->SetSizeHints(this);
 
-	int res = m_pDlg->ShowModal();
-
-	if (!displayOnly) {
-		if (res == wxID_OK) {
-			notification.m_trusted = true;
-
-			if (!notification.GetAlgorithmWarnings()) {
-				bool trustSANs = sanTrustAllowed && xrc_call(*m_pDlg, "ID_TRUST_SANS", &wxCheckBox::GetValue);
-				bool permanent = !warning && xrc_call(*m_pDlg, "ID_ALWAYS", &wxCheckBox::GetValue);
-				certStore_.SetTrusted(notification, permanent, trustSANs);
-			}
-		}
-		else {
-			notification.m_trusted = false;
-		}
-	}
-
-	delete m_pDlg;
-	m_pDlg = 0;
+	return true;
 }
 
 namespace {
@@ -700,18 +704,34 @@ void CVerifyCertDialog::OnCertificateChoice(wxCommandEvent& event)
 	if (sel < 0 || static_cast<unsigned int>(sel) > m_certificates.size()) {
 		return;
 	}
-	DisplayCert(m_pDlg, m_certificates[sel]);
+	DisplayCert(m_certificates[sel]);
 
-	m_pDlg->Layout();
-	m_pDlg->GetSizer()->Fit(m_pDlg);
-	m_pDlg->Refresh();
+	Layout();
+	GetSizer()->Fit(this);
+	Refresh();
 }
 
 
-void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification & notification)
+void ConfirmInsecureConection(wxWindow* parent, CertStore & certStore, CInsecureConnectionNotification & notification)
 {
 	wxDialogEx dlg;
-	dlg.Create(0, wxID_ANY, _("Insecure FTP connection"));
+
+	auto const protocol = notification.server_.GetProtocol();
+
+	wxString name;
+	switch (protocol) {
+	case FTP:
+	case INSECURE_FTP:
+		name = L"FTP";
+		break;
+	case INSECURE_WEBDAV:
+		name = L"WebDAV";
+		break;
+	default:
+		name = CServer::GetProtocolName(protocol);
+		break;
+	}
+	dlg.Create(parent, wxID_ANY, wxString::Format(_("Insecure %s connection"), name));
 
 	auto const& lay = dlg.layout();
 	auto outer = new wxBoxSizer(wxVERTICAL);
@@ -720,14 +740,24 @@ void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification &
 	auto main = lay.createFlex(1);
 	outer->Add(main, 0, wxALL, lay.border);
 
-	bool const warning = certStore.HasCertificate(notification.server_.GetHost(), notification.server_.GetPort());
+	bool const warning = certStore.HasCertificate(fz::to_utf8(notification.server_.GetHost()), notification.server_.GetPort());
 
 	if (warning) {
-		main->Add(new wxStaticText(&dlg, -1, _("Warning! You have previously connected to this server using FTP over TLS, yet the server has now rejected FTP over TLS.")));
-		main->Add(new wxStaticText(&dlg, -1, _("This may be the result of a downgrade attack, only continue after you have spoken to the server administrator or server hosting provider.")));
+		if (protocol == FTP) {
+			main->Add(new wxStaticText(&dlg, -1, _("Warning! You have previously connected to this server using FTP over TLS, yet the server has now rejected FTP over TLS.")));
+			main->Add(new wxStaticText(&dlg, -1, _("This may be the result of a downgrade attack, only continue after you have spoken to the server administrator or server hosting provider.")));
+		}
+		else {
+			main->Add(new wxStaticText(&dlg, -1, _("Warning! You have previously connected to this server using TLS.")));
+		}
 	}
 	else {
-		main->Add(new wxStaticText(&dlg, -1, _("This server does not support FTP over TLS.")));
+		if (protocol == FTP) {
+			main->Add(new wxStaticText(&dlg, -1, _("This server does not support FTP over TLS.")));
+		}
+		else {
+			main->Add(new wxStaticText(&dlg, -1, wxString::Format(_("Using plain %s is insecure."), name)));
+		}
 	}
 	main->Add(new wxStaticText(&dlg, -1, _("If you continue, your password and files will be sent in clear over the internet.")));
 
@@ -739,7 +769,7 @@ void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification &
 	flex->Add(new wxStaticText(&dlg, -1, _("Port:")), lay.valign);
 	flex->Add(new wxStaticText(&dlg, -1, fz::to_wstring(notification.server_.GetPort())), lay.valign);
 
-	auto always = new wxCheckBox(&dlg, -1, _("&Always allow insecure plain FTP for this server."));
+	auto always = new wxCheckBox(&dlg, -1, wxString::Format(_("&Always allow insecure plain %s for this server."), name));
 	main->Add(always);
 
 	auto buttons = lay.createButtonSizer(&dlg, main, true);
@@ -758,7 +788,9 @@ void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification &
 	}
 	buttons->AddButton(cancel);
 
-	dlg.Bind(wxEVT_BUTTON, [&dlg](wxEvent & evt) {dlg.EndModal(evt.GetId()); });
+	auto onButton = [&dlg](wxEvent & evt) {dlg.EndModal(evt.GetId()); };
+	ok->Bind(wxEVT_BUTTON, onButton);
+	cancel->Bind(wxEVT_BUTTON, onButton);
 
 	buttons->Realize();
 
@@ -773,6 +805,6 @@ void ConfirmInsecureConection(CertStore & certStore, CInsecureFTPNotification &
 	if (allow) {
 		notification.allow_ = true;
 
-		certStore.SetInsecure(notification.server_.GetHost(), notification.server_.GetPort(), always->GetValue());
+		certStore.SetInsecure(fz::to_utf8(notification.server_.GetHost()), notification.server_.GetPort(), always->GetValue());
 	}
 }
